@@ -87,6 +87,8 @@ struct _CalendarWindowPrivate {
 	gboolean     show_weeks;
 	time_t      *current_time;
 
+	gboolean     locked_down;
+
 	GtkWidget *locations_list;
 
 #ifdef HAVE_LIBECAL
@@ -123,7 +125,8 @@ enum {
 	PROP_TIMEFORMAT,
 #endif
 	PROP_CURRENTTIMEP,
-	PROP_PREFSDIR
+	PROP_PREFSDIR,
+	PROP_LOCKEDDOWN
 };
 
 static time_t *calendar_window_get_current_time_p (CalendarWindow *calwin);
@@ -132,11 +135,15 @@ static void    calendar_window_set_current_time_p (CalendarWindow *calwin,
 static const char *calendar_window_get_prefs_dir  (CalendarWindow *calwin);
 static void    calendar_window_set_prefs_dir      (CalendarWindow *calwin,
 						   const char     *prefs_dir);
+static gboolean calendar_window_get_locked_down   (CalendarWindow *calwin);
+static void    calendar_window_set_locked_down    (CalendarWindow *calwin,
+						   gboolean        locked_down);
 static GtkWidget * create_hig_frame 		  (CalendarWindow *calwin,
 		  				   const char *title,
                   				   const char *button_label,
 		  				   const char *key,
-                  				   GCallback   callback);
+						   GCallback   callback,
+						   gboolean    bind_to_locked_down);
 
 #ifdef HAVE_LIBECAL
 
@@ -260,7 +267,7 @@ update_frame_visibility (GtkWidget    *frame,
 
 enum {
         APPOINTMENT_COLUMN_UID,
-        APPOINTMENT_COLUMN_URI,
+        APPOINTMENT_COLUMN_TYPE,
         APPOINTMENT_COLUMN_SUMMARY,
         APPOINTMENT_COLUMN_DESCRIPTION,
         APPOINTMENT_COLUMN_START_TIME,
@@ -271,6 +278,12 @@ enum {
         N_APPOINTMENT_COLUMNS
 };
 
+enum {
+	APPOINTMENT_TYPE_APPOINTMENT,
+	APPOINTMENT_TYPE_BIRTHDAY,
+	APPOINTMENT_TYPE_WEATHER
+};
+
 enum {
         TASK_COLUMN_UID,
         TASK_COLUMN_SUMMARY,
@@ -454,49 +467,14 @@ handle_task_percent_complete_edited (CalendarWindow      *calwin,
 }
 
 static gboolean
-is_appointment (GtkTreeModel *model,
-		GtkTreeIter  *iter,
-		gpointer      data)
+is_for_filter (GtkTreeModel *model,
+	      GtkTreeIter  *iter,
+	      gpointer      data)
 {
-	gchar *uri;
+	gint type;
 
-	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_URI, &uri, -1);
-	if (uri)
-		return (g_ascii_strcasecmp (uri, "file")   == 0 ||
-			g_ascii_strcasecmp (uri, "local")   == 0 ||
-			g_ascii_strcasecmp (uri, "webcal") == 0 ||
-			g_ascii_strcasecmp (uri, "caldav") == 0 ||
-			g_ascii_strcasecmp (uri, "mapi")   == 0 ||
-			g_ascii_strcasecmp (uri, "exchange")  == 0 ||
-			g_ascii_strcasecmp (uri, "groupwise") == 0 ||
-			g_ascii_strcasecmp (uri, "google") == 0);
-	return FALSE;
-}
-
-static gboolean
-is_birthday (GtkTreeModel *model,
-	     GtkTreeIter  *iter,
-	     gpointer      data)
-{
-	gchar *uri;
-
-	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_URI, &uri, -1);
-	if (uri)
-		return (g_ascii_strcasecmp (uri, "contacts") == 0);
-	return FALSE;
-}
-
-static gboolean
-is_weather (GtkTreeModel *model,
-	    GtkTreeIter  *iter,
-	    gpointer      data)
-{
-	gchar *uri;
-
-	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_URI, &uri, -1);
-	if (uri)
-		return (g_ascii_strcasecmp (uri, "weather") == 0);
-	return FALSE;
+	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_TYPE, &type, -1);
+	return type == GPOINTER_TO_INT (data);
 }
 
 static gboolean
@@ -861,7 +839,8 @@ create_task_list (CalendarWindow *calwin,
 	list = create_hig_frame (calwin, 
                                  _("Tasks"), _("Edit"),
                                  KEY_TASKS_EXPANDED,
-                                 G_CALLBACK (edit_tasks)); 
+                                 G_CALLBACK (edit_tasks),
+				 FALSE);
  
         *scrolled_window = scrolled = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
@@ -976,6 +955,7 @@ handle_appointments_changed (CalendarWindow *calwin)
                 CalendarAppointment *appointment = l->data;
                 GtkTreeIter          iter;
                 char                *start_text;
+                gint                 type;
 
                 g_assert (CALENDAR_EVENT (appointment)->type == CALENDAR_EVENT_APPOINTMENT);
 
@@ -986,12 +966,18 @@ handle_appointments_changed (CalendarWindow *calwin)
                                                   appointment->start_time,
                                                   year, month, day);
 
+                if (g_ascii_strcasecmp (appointment->uri, "weather") == 0)
+                        type = APPOINTMENT_TYPE_WEATHER;
+                else if (g_ascii_strcasecmp (appointment->uri, "contacts") == 0)
+                        type = APPOINTMENT_TYPE_BIRTHDAY;
+                else
+                        type = APPOINTMENT_TYPE_APPOINTMENT;
 
                 gtk_list_store_append (calwin->priv->appointments_model,
 				       &iter);
                 gtk_list_store_set (calwin->priv->appointments_model, &iter,
                                     APPOINTMENT_COLUMN_UID,         appointment->uid,
-                                    APPOINTMENT_COLUMN_URI,         appointment->uri,
+                                    APPOINTMENT_COLUMN_TYPE,        type,
                                     APPOINTMENT_COLUMN_SUMMARY,     appointment->summary,
                                     APPOINTMENT_COLUMN_DESCRIPTION, appointment->description,
                                     APPOINTMENT_COLUMN_START_TIME,  (gint64)appointment->start_time,
@@ -1018,7 +1004,7 @@ static GtkWidget *
 create_list_for_appointment_model (CalendarWindow      *calwin,
 				   const char          *label,
 				   GtkTreeModelFilter **filter,
-				   GtkTreeModelFilterVisibleFunc is_for_filter,
+				   gint                 filter_type,
 				   GtkTreeCellDataFunc  set_pixbuf_cell,
 				   gboolean             show_start,
 				   GtkWidget          **tree_view,
@@ -1034,7 +1020,7 @@ create_list_for_appointment_model (CalendarWindow      *calwin,
 	GtkTreeSelection  *selection;
 
 	
-	list = create_hig_frame (calwin, label, _("Edit"), key, callback);
+	list = create_hig_frame (calwin, label, _("Edit"), key, callback, FALSE);
 
         *scrolled_window = scrolled = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
@@ -1058,7 +1044,7 @@ create_list_for_appointment_model (CalendarWindow      *calwin,
 		gtk_tree_model_filter_set_visible_func (
 				*filter,
 				(GtkTreeModelFilterVisibleFunc) is_for_filter,
-				calwin,
+				GINT_TO_POINTER (filter_type),
 				NULL);
         }
 
@@ -1120,7 +1106,7 @@ create_appointment_list (CalendarWindow  *calwin,
 					calwin,
 					_("Appointments"),
 					&calwin->priv->appointments_filter,
-					is_appointment,
+					APPOINTMENT_TYPE_APPOINTMENT,
 					appointment_pixbuf_cell_data_func,
 					TRUE,
 					tree_view,
@@ -1145,7 +1131,7 @@ create_birthday_list (CalendarWindow  *calwin,
 					calwin,
 					_("Birthdays and Anniversaries"),
 					&calwin->priv->birthdays_filter,
-					is_birthday,
+					APPOINTMENT_TYPE_BIRTHDAY,
 					birthday_pixbuf_cell_data_func,
 					FALSE,
 					tree_view,
@@ -1170,7 +1156,7 @@ create_weather_list (CalendarWindow  *calwin,
 					calwin,
 					_("Weather Information"),
 					&calwin->priv->weather_filter,
-					is_weather,
+					APPOINTMENT_TYPE_WEATHER,
 					weather_pixbuf_cell_data_func,
 					FALSE,
 					tree_view,
@@ -1232,7 +1218,7 @@ calendar_window_create_appointments_model (CalendarWindow *calwin)
 	calwin->priv->appointments_model =
 		gtk_list_store_new (N_APPOINTMENT_COLUMNS,
 				    G_TYPE_STRING,   /* uid              */
-				    G_TYPE_STRING,   /* uri              */
+				    G_TYPE_INT,      /* type             */
 				    G_TYPE_STRING,   /* summary          */
 				    G_TYPE_STRING,   /* description      */
 				    G_TYPE_INT64,    /* start time       */
@@ -1253,32 +1239,13 @@ calendar_day_activated (GtkCalendar    *calendar,
 	unsigned int  day;
 	unsigned int  month;
 	unsigned int  year;
-	time_t        date;
-	struct tm     utc_date_tm;
-	struct tm     local_date_tm = { 0, };
 	char         *argument;
 
 	gtk_calendar_get_date (calendar, &year, &month, &day);
 
-	local_date_tm.tm_mday  = (int) day;
-	local_date_tm.tm_mon   = (int) month;
-	local_date_tm.tm_year  = (int) year - 1900;
-	local_date_tm.tm_isdst = -1;
-
-	/* convert the local date picked in the calendar to broken-down UTC */
-	date = mktime (&local_date_tm);
-	gmtime_r (&date, &utc_date_tm);
-
-	/* FIXME: once bug 409200 is fixed, we'll have to make this hh:mm:ss
-	 * instead of hhmmss */
 	argument = g_strdup_printf ("calendar:///?startdate="
-				    "%.4d%.2d%.2dT%.2d%.2d%.2dZ",
-				    utc_date_tm.tm_year + 1900,
-				    utc_date_tm.tm_mon + 1,
-				    utc_date_tm.tm_mday,
-				    utc_date_tm.tm_hour,
-				    utc_date_tm.tm_min,
-				    0);
+				    "%.4d%.2d%.2d",
+				    year, month, day);
 
 	clock_launch_calendar_app (calwin, argument);
 
@@ -1585,7 +1552,8 @@ create_hig_frame (CalendarWindow *calwin,
 		  const char *title,
                   const char *button_label,
 		  const char *key,
-                  GCallback   callback)
+		  GCallback   callback,
+		  gboolean    bind_to_locked_down)
 {
         GtkWidget *vbox;
         GtkWidget *alignment;
@@ -1634,6 +1602,11 @@ create_hig_frame (CalendarWindow *calwin,
                 gtk_container_add (GTK_CONTAINER (hbox), alignment);
 
                 g_signal_connect_swapped (button, "clicked", callback, calwin);
+
+		if (bind_to_locked_down)
+			g_object_bind_property (calwin, "locked-down",
+						button, "visible",
+						G_BINDING_DEFAULT|G_BINDING_INVERT_BOOLEAN|G_BINDING_SYNC_CREATE);
         }
 
 #ifdef HAVE_LIBECAL
@@ -1655,7 +1628,8 @@ calendar_window_pack_locations (CalendarWindow *calwin, GtkWidget *vbox)
 	calwin->priv->locations_list = create_hig_frame (calwin,
 							 _("Locations"), _("Edit"),
 							 KEY_LOCATIONS_EXPANDED,
-							 G_CALLBACK (edit_locations));
+							 G_CALLBACK (edit_locations),
+							 TRUE);
 
 	/* we show the widget before adding to the container, since adding to
 	 * the container changes the visibility depending on the state of the
@@ -1761,6 +1735,10 @@ calendar_window_get_property (GObject    *object,
 		g_value_set_string (value,
 				    calendar_window_get_prefs_dir (calwin));
 		break;
+	case PROP_LOCKEDDOWN:
+		g_value_set_boolean (value,
+				     calendar_window_get_locked_down (calwin));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1802,6 +1780,10 @@ calendar_window_set_property (GObject       *object,
 		calendar_window_set_prefs_dir (calwin,
 					       g_value_get_string (value));
 		break;
+	case PROP_LOCKEDDOWN:
+		calendar_window_set_locked_down (calwin,
+						 g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1919,6 +1901,15 @@ calendar_window_class_init (CalendarWindowClass *klass)
 				     "Preferences directory in GConf",
 				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (
+		gobject_class,
+		PROP_LOCKEDDOWN,
+		g_param_spec_boolean ("locked-down",
+				      "Locked Down",
+				      "Whether the window should offer access to preferences",
+				      FALSE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static void
@@ -2122,3 +2113,25 @@ calendar_window_set_prefs_dir (CalendarWindow *calwin,
 
 	g_object_notify (G_OBJECT (calwin), "prefs-dir");
 }
+
+static gboolean
+calendar_window_get_locked_down (CalendarWindow *calwin)
+{
+	g_return_val_if_fail (CALENDAR_IS_WINDOW (calwin), FALSE);
+
+	return calwin->priv->locked_down;
+}
+
+static void
+calendar_window_set_locked_down (CalendarWindow *calwin,
+				 gboolean        locked_down)
+{
+	g_return_if_fail (CALENDAR_IS_WINDOW (calwin));
+
+	if (locked_down == calwin->priv->locked_down)
+		return;
+
+	calwin->priv->locked_down = locked_down;
+
+	g_object_notify (G_OBJECT (calwin), "locked-down");
+}
