@@ -14,34 +14,32 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 
+#include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-show.h>
 
 #include "button-widget.h"
-#include "drawer.h"
 #include "launcher.h"
+#include "panel.h"
 #include "panel-addto.h"
 #include "panel-bindings.h"
-#include "panel-gconf.h"
-#include "panel-config-global.h"
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
 #include "panel-separator.h"
-#include "panel-compatibility.h"
 #include "panel-toplevel.h"
 #include "panel-util.h"
-#include "panel-profile.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
 #include "panel-properties-dialog.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
-
-#define SMALL_ICON_SIZE 20
+#include "panel-schemas.h"
 
 static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
 
+static GtkWidget *panel_applet_get_menu (AppletInfo *info);
 static void applet_menu_show       (GtkWidget *w, AppletInfo *info);
 static void applet_menu_deactivate (GtkWidget *w, AppletInfo *info);
 
@@ -56,9 +54,6 @@ panel_applet_set_dnd_enabled (AppletInfo *info,
 			      gboolean    dnd_enabled)
 {
 	switch (info->type) {
-	case PANEL_OBJECT_DRAWER:
-		panel_drawer_set_dnd_enabled (info->data, dnd_enabled);
-		break;
 	case PANEL_OBJECT_MENU:
 		panel_menu_button_set_dnd_enabled (PANEL_MENU_BUTTON (info->widget),
 						   dnd_enabled);
@@ -68,8 +63,6 @@ panel_applet_set_dnd_enabled (AppletInfo *info,
 		break;
 	case PANEL_OBJECT_APPLET:
 		break;
-	case PANEL_OBJECT_LOGOUT:
-	case PANEL_OBJECT_LOCK:
 	case PANEL_OBJECT_ACTION:
 		panel_action_button_set_dnd_enabled (PANEL_ACTION_BUTTON (info->widget),
 						     dnd_enabled);
@@ -84,36 +77,6 @@ panel_applet_set_dnd_enabled (AppletInfo *info,
 
 }
 
-gboolean
-panel_applet_toggle_locked (AppletInfo *info)
-{
-	PanelWidget *panel_widget;
-	gboolean     locked;
-
-	panel_widget = panel_applet_get_panel_widget (info);
-	
-	locked = panel_widget_toggle_applet_locked (panel_widget, info->widget);
-
-	panel_applet_save_position (info, info->id, TRUE);
-	panel_applet_set_dnd_enabled (info, !locked);
-
-	return locked;
-}
-
-static void
-panel_applet_lock (GtkCheckMenuItem *menuitem,
-		   AppletInfo       *info)
-{
-	gboolean locked;
-
-	locked = panel_applet_toggle_locked (info);
-
-	gtk_check_menu_item_set_active (menuitem, locked);
-
-	if (info->move_item)
-		gtk_widget_set_sensitive (info->move_item, !locked);
-}
-
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
@@ -182,80 +145,12 @@ panel_applet_recreate_menu (AppletInfo *info)
 	panel_applet_get_menu (info);
 }
 
-static void
-panel_applet_recreate_edit_menu (AppletInfo *info)
-{
-	if (info->edit_menu) {
-		if (gtk_widget_get_visible (info->edit_menu))
-			gtk_menu_shell_deactivate (GTK_MENU_SHELL (info->edit_menu));
-
-		g_signal_handlers_disconnect_by_func (info->edit_menu,
-						      G_CALLBACK (applet_menu_show), info);
-		g_signal_handlers_disconnect_by_func (info->edit_menu,
-						      G_CALLBACK (applet_menu_deactivate), info);
-		g_object_unref (info->edit_menu);
-		info->edit_menu = NULL;
-	}
-
-	panel_applet_get_edit_menu (info);
-}
-
-static void
-panel_applet_recreate_menus (AppletInfo *info)
-{
-	panel_applet_recreate_menu (info);
-	panel_applet_recreate_edit_menu (info);
-}
-
-static void
-panel_applet_locked_change_notify (GConfClient *client,
-				   guint        cnxn_id,
-				   GConfEntry  *entry,
-				   GtkWidget   *applet)
-{
-	GConfValue  *value;
-	gboolean     locked;
-	gboolean     applet_locked;
-	AppletInfo  *info;
-	PanelWidget *panel_widget;
-
-	g_assert (applet != NULL);
-
-	info = (AppletInfo  *) g_object_get_data (G_OBJECT (applet),
-						  "applet_info");
-	if (info == NULL)
-		return;
-
-	value = gconf_entry_get_value (entry);
-	if (value == NULL || value->type != GCONF_VALUE_BOOL)
-		return;
-
-	locked = gconf_value_get_bool (value);
-
-	panel_widget = panel_applet_get_panel_widget (info);
-	applet_locked = panel_widget_get_applet_locked (panel_widget,
-							info->widget);
-
-	if ((locked && applet_locked) || !(locked || applet_locked))
-		return;
-
-	panel_applet_toggle_locked (info);
-
-	if (info->type == PANEL_OBJECT_APPLET)
-		panel_applet_frame_sync_menu_state (PANEL_APPLET_FRAME (info->widget));
-	else
-		panel_applet_recreate_edit_menu (info);
-}
-
 static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
 {
 
-	if (info->type == PANEL_OBJECT_DRAWER)
-		drawer_query_deletion (info->data);
-	else
-		panel_profile_delete_object (info);
+	panel_layout_delete_object (info->id);
 }
 
 static inline GdkScreen *
@@ -285,28 +180,11 @@ applet_callback_callback (GtkWidget      *widget,
 		else if (!strcmp (menu->name, "properties"))
 			launcher_properties (menu->info->data);
 		break;
-	case PANEL_OBJECT_DRAWER: 
-		if (strcmp (menu->name, "add") == 0) {
-			Drawer *drawer = menu->info->data;
-
-			panel_addto_present (GTK_MENU_ITEM (widget),
-					     panel_toplevel_get_panel_widget (drawer->toplevel));
-		} else if (strcmp (menu->name, "properties") == 0) {
-			Drawer *drawer = menu->info->data;
-
-			panel_properties_dialog_present (drawer->toplevel);
-		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help (screen,
-					 "user-guide", "gospanel-18", NULL);
-		}
-		break;
 	case PANEL_OBJECT_MENU:
 		panel_menu_button_invoke_menu (
 			PANEL_MENU_BUTTON (menu->info->widget), menu->name);
 		break;
 	case PANEL_OBJECT_ACTION:
-	case PANEL_OBJECT_LOGOUT:
-	case PANEL_OBJECT_LOCK:
 		panel_action_button_invoke_menu (
 			PANEL_ACTION_BUTTON (menu->info->widget), menu->name);
 		break;
@@ -526,7 +404,16 @@ panel_applet_create_bare_menu (AppletInfo *info)
 	return menu;
 }
 
-GtkWidget *
+static void
+panel_applet_menu_lockdown_changed (PanelLockdown *lockdown,
+				    gpointer       user_data)
+{
+	AppletInfo *info = user_data;
+
+	panel_applet_recreate_menu (info);
+}
+
+static GtkWidget *
 panel_applet_get_menu (AppletInfo *info)
 {
 	GtkWidget   *menu;
@@ -562,35 +449,63 @@ panel_applet_get_menu (AppletInfo *info)
 
 	info->menu = menu;
 
+	panel_lockdown_on_notify (panel_lockdown_get (),
+				  NULL,
+				  G_OBJECT (info->menu),
+				  panel_applet_menu_lockdown_changed,
+				  info);
+
 	return info->menu;
 }
 
-GtkWidget *
+static void
+panel_applet_edit_menu_lockdown_changed (PanelLockdown *lockdown,
+					 gpointer       user_data)
+{
+	AppletInfo *info = user_data;
+
+	if (!panel_lockdown_get_panels_locked_down (lockdown))
+		return;
+
+	if (info->edit_menu) {
+		if (gtk_widget_get_visible (info->edit_menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL (info->edit_menu));
+
+                g_signal_handlers_disconnect_by_func (info->edit_menu,
+						      G_CALLBACK (applet_menu_deactivate), info);
+		gtk_widget_destroy (info->edit_menu);
+		info->edit_menu = NULL;
+	}
+}
+
+static GtkWidget *
 panel_applet_get_edit_menu (AppletInfo *info)
 {
 	GtkWidget   *menu;
 	GtkWidget   *menuitem;
 	PanelWidget *panel_widget;
 	GtkWidget   *image;
-	gboolean     locked;
-	gboolean     lockable;
 	gboolean     movable;
 	gboolean     removable;
 
 	if (info->edit_menu)
 		return info->edit_menu;
 
-	if (panel_lockdown_get_locked_down ())
+	if (panel_lockdown_get_panels_locked_down_s ())
 		return NULL;
 
 	menu = panel_applet_create_bare_menu (info);
 	panel_widget = panel_applet_get_panel_widget (info);
 
-	lockable = panel_applet_lockable (info);
 	movable = panel_applet_can_freely_move (info);
-	removable = panel_profile_id_lists_are_writable ();
+	removable = panel_layout_is_writable ();
 
-	locked = panel_widget_get_applet_locked (panel_widget, info->widget);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Move"));
+	g_signal_connect (menuitem, "activate",
+			  G_CALLBACK (move_applet_callback), info);
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	gtk_widget_set_sensitive (menuitem, movable);
 
 	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Remove From Panel"));
 	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
@@ -601,36 +516,16 @@ panel_applet_get_edit_menu (AppletInfo *info)
 			  G_CALLBACK (applet_remove_callback), info);
 	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_set_sensitive (menuitem, (!locked || lockable) && removable);
-
-	menuitem = gtk_menu_item_new_with_mnemonic (_("_Move"));
-	g_signal_connect (menuitem, "activate",
-			  G_CALLBACK (move_applet_callback), info);
-	gtk_widget_show (menuitem);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_set_sensitive (menuitem, !locked && movable);
-
-	g_assert (info->move_item == NULL);
-
-	info->move_item = menuitem;
-	g_object_add_weak_pointer (G_OBJECT (menuitem),
-				   (gpointer *) &info->move_item);
-
-	menuitem = gtk_separator_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show (menuitem);
-
-	menuitem = gtk_check_menu_item_new_with_mnemonic (_("Loc_k To Panel"));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
-					locked);
-	g_signal_connect (menuitem, "toggled",
-			  G_CALLBACK (panel_applet_lock), info);
-	gtk_widget_show (menuitem);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_set_sensitive (menuitem, lockable);
+	gtk_widget_set_sensitive (menuitem, removable);
 
 	info->edit_menu = menu;
 
+	panel_lockdown_on_notify (panel_lockdown_get (),
+				  "panels-locked-down",
+				  G_OBJECT (info->edit_menu),
+				  panel_applet_edit_menu_lockdown_changed,
+				  info);
+
 	return info->edit_menu;
 }
 
@@ -845,26 +740,10 @@ panel_applet_destroy (GtkWidget  *widget,
 	queued_position_saves =
 		g_slist_remove (queued_position_saves, info);
 
-	if (info->type == PANEL_OBJECT_DRAWER) {
-		Drawer *drawer = info->data;
-
-		if (drawer->toplevel) {
-			PanelWidget *panel_widget;
-
-			panel_widget = panel_toplevel_get_panel_widget (
-							drawer->toplevel);
-			panel_widget->master_widget = NULL;
-
-			gtk_widget_destroy (GTK_WIDGET (drawer->toplevel));
-			drawer->toplevel = NULL;
-		}
-	}
-
-	if (info->type != PANEL_OBJECT_APPLET)
-		panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menus),
-					      info);
-
 	if (info->menu) {
+		if (gtk_widget_get_visible (info->menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL (info->menu));
+
                 g_signal_handlers_disconnect_by_func (info->menu,
 						      G_CALLBACK (applet_menu_show), info);
                 g_signal_handlers_disconnect_by_func (info->menu,
@@ -874,6 +753,9 @@ panel_applet_destroy (GtkWidget  *widget,
 	info->menu = NULL;
 
 	if (info->edit_menu) {
+		if (gtk_widget_get_visible (info->edit_menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL (info->edit_menu));
+
                 g_signal_handlers_disconnect_by_func (info->edit_menu,
 						      G_CALLBACK (applet_menu_show), info);
                 g_signal_handlers_disconnect_by_func (info->edit_menu,
@@ -899,307 +781,16 @@ panel_applet_destroy (GtkWidget  *widget,
 	g_list_free (info->user_menu);
 	info->user_menu = NULL;
 
+	if (info->settings)
+		g_object_unref (info->settings);
+	info->settings = NULL;
+
 	g_free (info->id);
 	info->id = NULL;
 
 	g_free (info);
 }
 
-typedef struct {
-	char            *id;
-	PanelObjectType  type;
-	char            *toplevel_id;
-	int              position;
-	guint            right_stick : 1;
-	guint            locked : 1;
-} PanelAppletToLoad;
-
-/* Each time those lists get both empty,
- * panel_applet_queue_initial_unhide_toplevels() should be called */
-static GSList  *panel_applets_to_load = NULL;
-static GSList  *panel_applets_loading = NULL;
-/* We have a timeout to always unhide toplevels after a delay, in case of some
- * blocking applet */
-#define         UNHIDE_TOPLEVELS_TIMEOUT_SECONDS 5
-static guint    panel_applet_unhide_toplevels_timeout = 0;
-
-static gboolean panel_applet_have_load_idle = FALSE;
-
-static void
-free_applet_to_load (PanelAppletToLoad *applet)
-{
-	g_free (applet->id);
-	applet->id = NULL;
-
-	g_free (applet->toplevel_id);
-	applet->toplevel_id = NULL;
-
-	g_free (applet);
-}
-
-gboolean
-panel_applet_on_load_queue (const char *id)
-{
-	GSList *li;
-	for (li = panel_applets_to_load; li != NULL; li = li->next) {
-		PanelAppletToLoad *applet = li->data;
-		if (strcmp (applet->id, id) == 0)
-			return TRUE;
-	}
-	for (li = panel_applets_loading; li != NULL; li = li->next) {
-		PanelAppletToLoad *applet = li->data;
-		if (strcmp (applet->id, id) == 0)
-			return TRUE;
-	}
-	return FALSE;
-}
-
-/* This doesn't do anything if the initial unhide already happened */
-static gboolean
-panel_applet_queue_initial_unhide_toplevels (gpointer user_data)
-{
-	GSList *l;
-
-	if (panel_applet_unhide_toplevels_timeout != 0) {
-		g_source_remove (panel_applet_unhide_toplevels_timeout);
-		panel_applet_unhide_toplevels_timeout = 0;
-	}
-
-	for (l = panel_toplevel_list_toplevels (); l != NULL; l = l->next)
-		panel_toplevel_queue_initial_unhide ((PanelToplevel *) l->data);
-
-	return FALSE;
-}
-
-void
-panel_applet_stop_loading (const char *id)
-{
-	PanelAppletToLoad *applet;
-	GSList *l;
-
-	for (l = panel_applets_loading; l; l = l->next) {
-		applet = l->data;
-
-		if (strcmp (applet->id, id) == 0)
-			break;
-	}
-
-	/* this can happen if we reload an applet after it crashed,
-	 * for example */
-	if (l != NULL) {
-		panel_applets_loading = g_slist_delete_link (panel_applets_loading, l);
-		free_applet_to_load (applet);
-	}
-
-	if (panel_applets_loading == NULL && panel_applets_to_load == NULL)
-		panel_applet_queue_initial_unhide_toplevels (NULL);
-}
-
-static gboolean
-panel_applet_load_idle_handler (gpointer dummy)
-{
-	PanelObjectType    applet_type;
-	PanelAppletToLoad *applet = NULL;
-	PanelToplevel     *toplevel = NULL;
-	PanelWidget       *panel_widget;
-	GSList            *l;
-
-	if (!panel_applets_to_load) {
-		panel_applet_have_load_idle = FALSE;
-		return FALSE;
-	}
-
-	for (l = panel_applets_to_load; l; l = l->next) {
-		applet = l->data;
-
-		toplevel = panel_profile_get_toplevel_by_id (applet->toplevel_id);
-		if (toplevel)
-			break;
-	}
-
-	if (!l) {
-		/* All the remaining applets don't have a panel */
-		for (l = panel_applets_to_load; l; l = l->next)
-			free_applet_to_load (l->data);
-		g_slist_free (panel_applets_to_load);
-		panel_applets_to_load = NULL;
-		panel_applet_have_load_idle = FALSE;
-
-		if (panel_applets_loading == NULL) {
-			/* unhide any potential initially hidden toplevel */
-			panel_applet_queue_initial_unhide_toplevels (NULL);
-		}
-
-		return FALSE;
-	}
-
-	panel_applets_to_load = g_slist_delete_link (panel_applets_to_load, l);
-	panel_applets_loading = g_slist_append (panel_applets_loading, applet);
-
-	panel_widget = panel_toplevel_get_panel_widget (toplevel);
-
-	if (applet->right_stick) {
-		if (!panel_widget->packed)
-			applet->position = panel_widget->size - applet->position;
-		else
-			applet->position = -1;
-	}
-
-	/* We load applets asynchronously, so we specifically don't call
-	 * panel_applet_stop_loading() for this type. However, in case of
-	 * failure during the load, we might call panel_applet_stop_loading()
-	 * synchronously, which will make us lose the content of the applet
-	 * variable. So we save the type to be sure we always ignore the
-	 * applets. */
-	applet_type = applet->type;
-
-	switch (applet_type) {
-	case PANEL_OBJECT_APPLET:
-		panel_applet_frame_load_from_gconf (
-					panel_widget,
-					applet->locked,
-					applet->position,
-					applet->id);
-		break;
-	case PANEL_OBJECT_DRAWER:
-		drawer_load_from_gconf (panel_widget,
-					applet->locked,
-					applet->position,
-					applet->id);
-		break;
-	case PANEL_OBJECT_MENU:
-		panel_menu_button_load_from_gconf (panel_widget,
-						   applet->locked,
-						   applet->position,
-						   TRUE,
-						   applet->id);
-		break;
-	case PANEL_OBJECT_LAUNCHER:
-		launcher_load_from_gconf (panel_widget,
-					  applet->locked,
-					  applet->position,
-					  applet->id);
-		break;
-	case PANEL_OBJECT_LOGOUT:
-	case PANEL_OBJECT_LOCK:
-		panel_action_button_load_compatible (
-				applet->type,
-				panel_widget,
-				applet->locked,
-				applet->position,
-				TRUE,
-				applet->id);
-		break;
-	case PANEL_OBJECT_ACTION:
-		panel_action_button_load_from_gconf (
-				panel_widget,
-				applet->locked,
-				applet->position,
-				TRUE,
-				applet->id);
-		break;
-	case PANEL_OBJECT_MENU_BAR:
-		panel_menu_bar_load_from_gconf (
-				panel_widget,
-				applet->locked,
-				applet->position,
-				TRUE,
-				applet->id);
-		break;
-	case PANEL_OBJECT_SEPARATOR:
-		panel_separator_load_from_gconf (panel_widget,
-						 applet->locked,
-						 applet->position,
-						 applet->id);
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-
-	/* Only the real applets will do a late stop_loading */
-	if (applet_type != PANEL_OBJECT_APPLET)
-		panel_applet_stop_loading (applet->id);
-
-	return TRUE;
-}
-
-void
-panel_applet_queue_applet_to_load (const char      *id,
-				   PanelObjectType  type,
-				   const char      *toplevel_id,
-				   int              position,
-				   gboolean         right_stick,
-				   gboolean         locked)
-{
-	PanelAppletToLoad *applet;
-
-	if (!toplevel_id) {
-		g_warning ("No toplevel on which to load object '%s'\n", id);
-		return;
-	}
-
-	applet = g_new0 (PanelAppletToLoad, 1);
-
-	applet->id          = g_strdup (id);
-	applet->type        = type;
-	applet->toplevel_id = g_strdup (toplevel_id);
-	applet->position    = position;
-	applet->right_stick = right_stick != FALSE;
-	applet->locked      = locked != FALSE;
-
-	panel_applets_to_load = g_slist_prepend (panel_applets_to_load, applet);
-}
-
-static int
-panel_applet_compare (const PanelAppletToLoad *a,
-		      const PanelAppletToLoad *b)
-{
-	int c;
-
-	if ((c = strcmp (a->toplevel_id, b->toplevel_id)))
-		return c;
-	else if (a->right_stick != b->right_stick)
-		return b->right_stick ? -1 : 1;
-	else
-		return a->position - b->position;
-}
-
-void
-panel_applet_load_queued_applets (gboolean initial_load)
-{
-	if (!panel_applets_to_load) {
-		panel_applet_queue_initial_unhide_toplevels (NULL);
-		return;
-	}
-
-	if (panel_applets_to_load && panel_applet_unhide_toplevels_timeout == 0) {
-		/* Install a timeout to make sure we don't block the
-		 * unhiding because of an applet that doesn't load */
-		panel_applet_unhide_toplevels_timeout =
-			g_timeout_add_seconds (UNHIDE_TOPLEVELS_TIMEOUT_SECONDS,
-					       panel_applet_queue_initial_unhide_toplevels,
-					       NULL);
-	}
-
-	panel_applets_to_load = g_slist_sort (panel_applets_to_load,
-					      (GCompareFunc) panel_applet_compare);
-
-	if ( ! panel_applet_have_load_idle) {
-		/* on panel startup, we don't care about redraws of the
-		 * toplevels since they are hidden, so we give a higher
-		 * priority to loading of applets */
-		if (initial_load)
-			g_idle_add_full (G_PRIORITY_HIGH_IDLE,
-					 panel_applet_load_idle_handler,
-					 NULL, NULL);
-		else
-			g_idle_add (panel_applet_load_idle_handler, NULL);
-
-		panel_applet_have_load_idle = TRUE;
-	}
-}
-
 static G_CONST_RETURN char *
 panel_applet_get_toplevel_id (AppletInfo *applet)
 {
@@ -1212,7 +803,7 @@ panel_applet_get_toplevel_id (AppletInfo *applet)
 	if (!panel_widget)
 		return NULL;
 
-	return panel_profile_get_toplevel_id (panel_widget->toplevel);
+	return panel_toplevel_get_id (panel_widget->toplevel);
 }
 
 static gboolean
@@ -1239,15 +830,10 @@ panel_applet_save_position (AppletInfo *applet_info,
 			    const char *id,
 			    gboolean    immediate)
 {
-	PanelGConfKeyType  key_type;
-	GConfClient       *client;
-	PanelWidget       *panel_widget;
-	const char        *key;
-	const char        *toplevel_id;
-	char              *old_toplevel_id;
-	gboolean           right_stick;
-	gboolean           locked;
-	int                position;
+	PanelWidget *panel_widget;
+	const char  *toplevel_id;
+	gboolean     right_stick;
+	int          position;
 
 	g_return_if_fail (applet_info != NULL);
 
@@ -1268,46 +854,23 @@ panel_applet_save_position (AppletInfo *applet_info,
 	if (!(toplevel_id = panel_applet_get_toplevel_id (applet_info)))
 		return;
 
-	client  = panel_gconf_get_client ();
-
-	key_type = applet_info->type == PANEL_OBJECT_APPLET ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
-	
 	panel_widget = panel_applet_get_panel_widget (applet_info);
 
-	/* FIXME: Instead of getting keys, comparing and setting, there
-	   should be a dirty flag */
-
-	key = panel_gconf_full_key (key_type, id, "toplevel_id");
-	old_toplevel_id = gconf_client_get_string (client, key, NULL);
-	if (old_toplevel_id == NULL || strcmp (old_toplevel_id, toplevel_id) != 0)
-		gconf_client_set_string (client, key, toplevel_id, NULL);
-	g_free (old_toplevel_id);
-
-	/* Note: changing some properties of the panel that may not be locked down
-	   (e.g. background) can change the state of the "panel_right_stick" and
-	   "position" properties of an applet that may in fact be locked down.
-	   So check if these are writable before attempting to write them */
-
 	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
-	key = panel_gconf_full_key (
-			key_type, id, "panel_right_stick");
-	if (gconf_client_key_is_writable (client, key, NULL) &&
-	    (gconf_client_get_bool (client, key, NULL) ? 1 : 0) != right_stick)
-		gconf_client_set_bool (client, key, right_stick, NULL);
 
 	position = panel_applet_get_position (applet_info);
 	if (right_stick && !panel_widget->packed)
 		position = panel_widget->size - position;
 
-	key = panel_gconf_full_key (key_type, id, "position");
-	if (gconf_client_key_is_writable (client, key, NULL) &&
-	    gconf_client_get_int (client, key, NULL) != position)
-		gconf_client_set_int (client, key, position, NULL);
-	
-	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
-	key = panel_gconf_full_key (key_type, id, "locked");
-	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != locked)
-		gconf_client_set_bool (client, key, locked, NULL);
+	g_settings_set_string (applet_info->settings,
+			       PANEL_OBJECT_TOPLEVEL_ID_KEY,
+			       toplevel_id);
+	g_settings_set_enum (applet_info->settings,
+			     PANEL_OBJECT_PACK_KEY,
+			     right_stick ? PANEL_OBJECT_PACK_END : PANEL_OBJECT_PACK_START);
+	g_settings_set_int (applet_info->settings,
+			    PANEL_OBJECT_POSITION_KEY,
+			    position);
 }
 
 const char *
@@ -1380,17 +943,16 @@ panel_applet_get_by_type (PanelObjectType object_type, GdkScreen *screen)
 
 AppletInfo *
 panel_applet_register (GtkWidget       *applet,
-		       gpointer         data,
-		       GDestroyNotify   data_destroy,
 		       PanelWidget     *panel,
-		       gboolean         locked,
-		       gint             pos,
-		       gboolean         exactpos,
 		       PanelObjectType  type,
-		       const char      *id)
+		       const char      *id,
+		       GSettings       *settings,
+		       gpointer         data,
+		       GDestroyNotify   data_destroy)
 {
-	AppletInfo *info;
-	const char *key;
+	AppletInfo      *info;
+	int              pos;
+	PanelObjectPack  pack;
 	
 	g_return_val_if_fail (applet != NULL && panel != NULL, NULL);
 
@@ -1403,59 +965,43 @@ panel_applet_register (GtkWidget       *applet,
 	info = g_new0 (AppletInfo, 1);
 	info->type         = type;
 	info->widget       = applet;
+	info->settings     = g_object_ref (settings);
 	info->menu         = NULL;
 	info->edit_menu    = NULL;
 	info->data         = data;
 	info->data_destroy = data_destroy;
 	info->user_menu    = NULL;
-	info->move_item    = NULL;
 	info->id           = g_strdup (id);
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
-	if (type != PANEL_OBJECT_APPLET)
-		panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menus),
-					   info);
-
-	key = panel_gconf_full_key ((type == PANEL_OBJECT_APPLET) ?
-				     PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS,
-				    id, "locked");
-	panel_gconf_notify_add_while_alive (key,
-					    (GConfClientNotifyFunc) panel_applet_locked_change_notify,
-					    G_OBJECT (applet));
-
-	if (type == PANEL_OBJECT_DRAWER) {
-		Drawer *drawer = data;
-		PanelWidget *assoc_panel;
-
-		assoc_panel = panel_toplevel_get_panel_widget (drawer->toplevel);
-
-		g_object_set_data (G_OBJECT (applet),
-				   PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
-		assoc_panel->master_widget = applet;
-		g_object_add_weak_pointer (
-			G_OBJECT (applet), (gpointer *) &assoc_panel->master_widget);
-	}
-
-	g_object_set_data (G_OBJECT (applet),
-			   PANEL_APPLET_FORBIDDEN_PANELS, NULL);
-
 	registered_applets = g_slist_append (registered_applets, info);
 
-	if (panel_widget_add (panel, applet, locked, pos, exactpos) == -1 &&
-	    panel_widget_add (panel, applet, locked, 0, TRUE) == -1) {
+	/* Find where to insert the applet */
+        pos = g_settings_get_int (info->settings, PANEL_OBJECT_POSITION_KEY);
+        pack = g_settings_get_enum (info->settings, PANEL_OBJECT_PACK_KEY);
+        if (pack == PANEL_OBJECT_PACK_END) {
+                if (!panel->packed)
+                        pos = panel->size - pos;
+                else
+                        pos = -1;
+        }
+
+	/* Insert it */
+	if (panel_widget_add (panel, applet, pos, TRUE) == -1 &&
+	    panel_widget_add (panel, applet, 0, FALSE) == -1) {
 		GSList *l;
 
 		for (l = panels; l; l = l->next) {
 			panel = PANEL_WIDGET (l->data);
 
-			if (panel_widget_add (panel, applet, locked, 0, TRUE) != -1)
+			if (panel_widget_add (panel, applet, 0, FALSE) != -1)
 				break;
 		}
 
 		if (!l) {
 			g_warning (_("Cannot find an empty spot"));
-			panel_profile_delete_object (info);
+			panel_layout_delete_object (info->id);
 			return NULL;
 		}
 	}
@@ -1475,9 +1021,9 @@ panel_applet_register (GtkWidget       *applet,
 			  G_CALLBACK (panel_applet_destroy),
 			  info);
 
-	panel_applet_set_dnd_enabled (info, !locked);
+	panel_applet_set_dnd_enabled (info, TRUE);
 
-	gtk_widget_show_all (applet);
+	gtk_widget_show (applet);
 
 	orientation_change (info, panel);
 	size_change (info, panel);
@@ -1504,50 +1050,26 @@ panel_applet_get_position (AppletInfo *applet)
 	return applet_data->pos;
 }
 
+GSettings *
+panel_applet_get_settings (AppletInfo *applet)
+{
+	g_return_val_if_fail (applet != NULL, NULL);
+
+	return applet->settings;
+}
+
 gboolean
 panel_applet_can_freely_move (AppletInfo *applet)
 {
-	GConfClient       *client;
-	PanelGConfKeyType  key_type;
-	const char        *key;
-
-	if (panel_lockdown_get_locked_down ())
+	/* if we check for more lockdown than this, then we'll need to update
+	 * callers that use panel_lockdown_on_notify() */
+	if (panel_lockdown_get_panels_locked_down_s ())
 		return FALSE;
 
-	client  = panel_gconf_get_client ();
-	
-	key_type = (applet->type == PANEL_OBJECT_APPLET) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
-       
-	key = panel_gconf_full_key (key_type, applet->id, "position");
-	if (!gconf_client_key_is_writable (client, key, NULL))
-		return FALSE;
-
-	key = panel_gconf_full_key (key_type, applet->id, "toplevel_id");
-	if (!gconf_client_key_is_writable (client, key, NULL))
-		return FALSE;
-
-	key = panel_gconf_full_key (key_type, applet->id, "panel_right_stick");
-	if (!gconf_client_key_is_writable (client, key, NULL))
-		return FALSE;
-
-	return TRUE;
-}
-
-gboolean
-panel_applet_lockable (AppletInfo *applet)
-{
-	GConfClient        *client;
-	PanelGConfKeyType   key_type;
-	const char         *key;
-
-	if (panel_lockdown_get_locked_down ())
-		return FALSE;
-	
-	client  = panel_gconf_get_client ();
-	
-	key_type = (applet->type == PANEL_OBJECT_APPLET) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
-
-	key = panel_gconf_full_key (key_type, applet->id, "locked");
-
-	return gconf_client_key_is_writable (client, key, NULL);
+	return (g_settings_is_writable (applet->settings,
+					PANEL_OBJECT_TOPLEVEL_ID_KEY) &&
+	        g_settings_is_writable (applet->settings,
+					PANEL_OBJECT_POSITION_KEY) &&
+	        g_settings_is_writable (applet->settings,
+					PANEL_OBJECT_PACK_KEY));
 }
