@@ -32,18 +32,16 @@
 
 #include "button-widget.h"
 #include "panel-util.h"
-#include "panel-config-global.h"
-#include "panel-gconf.h"
-#include "panel-profile.h"
 #include "xstuff.h"
 #include "panel-toplevel.h"
 #include "panel-a11y.h"
 #include "panel-globals.h"
 #include "panel-multiscreen.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
-#include "panel-compatibility.h"
 #include "panel-ditem-editor.h"
 #include "panel-icon-names.h"
+#include "panel-schemas.h"
 
 static GdkScreen *
 launcher_get_screen (Launcher *launcher)
@@ -96,6 +94,26 @@ launcher_register_error_dialog (Launcher *launcher,
 			  launcher);
 }
 
+static void
+launcher_do_zoom_animation (GtkWidget *widget)
+{
+	GtkSettings *settings;
+	gboolean     enable_animations;
+
+	settings = gtk_widget_get_settings (widget);
+
+	enable_animations = TRUE;
+	g_object_get (settings,
+		      "gtk-enable-animations", &enable_animations,
+		      NULL);
+
+	if (enable_animations)
+		xstuff_zoom_animate (widget,
+				     button_widget_get_pixbuf (BUTTON_WIDGET (widget)),
+				     button_widget_get_orientation (BUTTON_WIDGET (widget)),
+				     NULL);
+}
+
 static void
 launch_url (Launcher *launcher)
 {
@@ -136,11 +154,7 @@ launcher_launch (Launcher  *launcher,
 	g_return_if_fail (launcher != NULL);
 	g_return_if_fail (launcher->key_file != NULL);
 
-	if (panel_global_config_get_enable_animations ())
-		xstuff_zoom_animate (widget,
-				     button_widget_get_pixbuf (BUTTON_WIDGET (widget)),
-				     button_widget_get_orientation (BUTTON_WIDGET (widget)),
-				     NULL);
+	launcher_do_zoom_animation (widget);
 	
 	type = panel_key_file_get_string (launcher->key_file, "Type");
 	if (type && !strcmp (type, "Link"))
@@ -165,24 +179,6 @@ launcher_launch (Launcher  *launcher,
 		}
 	}
 	g_free (type);
-	
-	if (panel_global_config_get_drawer_auto_close ()) {
-		PanelToplevel *toplevel;
-		PanelToplevel *parent;
-
-		toplevel = PANEL_WIDGET (gtk_widget_get_parent (launcher->button))->toplevel;
-
-		if (panel_toplevel_get_is_attached (toplevel)) {
-			parent = panel_toplevel_get_attach_toplevel (toplevel);
-
-			while (panel_toplevel_get_is_attached (parent)) {
-				toplevel = parent;
-				parent = panel_toplevel_get_attach_toplevel (toplevel);
-			}
-
-			panel_toplevel_hide (toplevel, FALSE, -1);
-		}
-	}
 }
 
 static void
@@ -200,11 +196,7 @@ drag_data_received_cb (GtkWidget        *widget,
 	int      i;
 	GList   *file_list;
 
-	if (panel_global_config_get_enable_animations ())
-		xstuff_zoom_animate (widget,
-				     button_widget_get_pixbuf (BUTTON_WIDGET (widget)),
-				     button_widget_get_orientation (BUTTON_WIDGET (widget)),
-				     NULL);
+	launcher_do_zoom_animation (widget);
 	
 	file_list = NULL;
 	uris = g_uri_list_extract_uris ((const char *) gtk_selection_data_get_data (selection_data));
@@ -687,22 +679,19 @@ static void
 launcher_saved (GtkWidget *dialog,
 		Launcher  *launcher)
 {
-	const char  *uri;
-	GConfClient *client;
-	const char  *key;
+	const char *uri;
 
 	uri = panel_ditem_editor_get_uri (PANEL_DITEM_EDITOR (dialog));
 	if (panel_launcher_get_filename (uri) != NULL)
 		uri = panel_launcher_get_filename (uri);
 
 	if (uri && launcher->location && strcmp (uri, launcher->location)) {
-		client = panel_gconf_get_client ();
+		GSettings *settings;
 
-		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS,
-					    launcher->info->id,
-					    "launcher_location");
-
-		gconf_client_set_string (client, key, uri, NULL);
+		settings = panel_layout_get_instance_settings (launcher->info->settings,
+							       PANEL_LAUNCHER_SCHEMA);
+		g_settings_set_string (settings, PANEL_LOCATION_KEY, uri);
+		g_object_unref (settings);
 
 		if (launcher->location)
 			g_free (launcher->location);
@@ -766,20 +755,18 @@ launcher_properties (Launcher  *launcher)
 static gboolean
 lancher_properties_enabled (void)
 {
-	if (panel_lockdown_get_locked_down () ||
-	    panel_lockdown_get_disable_command_line ())
+	if (panel_lockdown_get_panels_locked_down_s () ||
+	    panel_lockdown_get_disable_command_line_s ())
 		return FALSE;
 
 	return TRUE;
 }
 
 static Launcher *
-load_launcher_applet (const char       *location,
-		      PanelWidget      *panel,
-		      gboolean          locked,
-		      int               pos,
-		      gboolean          exactpos,
-		      const char       *id)
+load_launcher_applet (const char  *location,
+		      PanelWidget *panel,
+		      const char  *id,
+		      GSettings   *settings)
 {
 	Launcher *launcher;
 
@@ -788,10 +775,10 @@ load_launcher_applet (const char       *location,
 	if (!launcher)
 		return NULL;
 
-	launcher->info = panel_applet_register (launcher->button, launcher,
-						free_launcher,
-						panel, locked, pos, exactpos,
-						PANEL_OBJECT_LAUNCHER, id);
+	launcher->info = panel_applet_register (launcher->button, panel,
+						PANEL_OBJECT_LAUNCHER, id,
+						settings,
+						launcher, free_launcher);
 	if (!launcher->info) {
 		free_launcher (launcher);
 		return NULL;
@@ -819,41 +806,37 @@ load_launcher_applet (const char       *location,
 }
 
 void
-launcher_load_from_gconf (PanelWidget *panel_widget,
-			  gboolean     locked,
-			  int          position,
-			  const char  *id)
+launcher_load (PanelWidget *panel_widget,
+	       const char  *id,
+	       GSettings   *settings)
 {
-	GConfClient *client;
-	Launcher    *launcher;
-	const char  *key;
-	char        *launcher_location;
+	GSettings *settings_instance;
+	Launcher  *launcher;
+	char      *launcher_location;
 
 	g_return_if_fail (panel_widget != NULL);
 	g_return_if_fail (id != NULL);
 
-	client  = panel_gconf_get_client ();
+	settings_instance = panel_layout_get_instance_settings (settings,
+								PANEL_LAUNCHER_SCHEMA);
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
-	panel_compatibility_migrate_applications_scheme (client, key);
-	launcher_location = gconf_client_get_string (client, key, NULL);
+	launcher_location = g_settings_get_string (settings_instance,
+						   PANEL_LOCATION_KEY);
 
-	if (!launcher_location) {
-		g_printerr (_("Key %s is not set, cannot load launcher\n"),
-			    key);
+	if (PANEL_GLIB_STR_EMPTY (launcher_location)) {
+		g_printerr (_("Launcher location is not set, cannot load launcher\n"));
+		g_free (launcher_location);
+		g_object_unref (settings_instance);
 		return;
 	}
 
 	launcher = load_launcher_applet (launcher_location,
 					 panel_widget,
-					 locked,
-					 position,
-					 TRUE,
-					 id);
+					 id, settings);
 
 	if (launcher) {
-		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
-		if (!gconf_client_key_is_writable (client, key, NULL)) {
+		if (!g_settings_is_writable (settings_instance,
+					     PANEL_LOCATION_KEY)) {
 			AppletUserMenu *menu;
 
 			menu = panel_applet_get_callback (launcher->info->user_menu,
@@ -864,6 +847,7 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	}
 
 	g_free (launcher_location);
+	g_object_unref (settings_instance);
 }
 
 static void
@@ -886,13 +870,12 @@ launcher_new_saved (GtkWidget *dialog,
 void
 ask_about_launcher (const char  *file,
 		    PanelWidget *panel,
-		    int          pos,
-		    gboolean     exactpos)
+		    int          pos)
 {
 	GtkWidget *dialog;
 	GKeyFile  *key_file;
 
-	if (panel_lockdown_get_disable_command_line ())
+	if (panel_lockdown_get_disable_command_line_s ())
 		return;
 
 	dialog = panel_ditem_editor_new (NULL, NULL, NULL,
@@ -978,20 +961,18 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 			       int            position,
 			       const char    *location)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *id;
-	char        *no_uri;
-	const char  *new_location;
+	char       *id;
+	GSettings  *settings;
+	GSettings  *settings_instance;
+	char       *no_uri;
+	const char *new_location;
 
 	g_return_if_fail (location != NULL);
 
-	client = panel_gconf_get_client ();
-
-	id = panel_profile_prepare_object_with_id (PANEL_OBJECT_LAUNCHER,
-						   toplevel_id,
-						   position,
-						   FALSE);
+	id = panel_layout_object_create_start (PANEL_OBJECT_LAUNCHER,
+					       NULL,
+					       toplevel_id, position, FALSE,
+					       &settings);
 
 	no_uri = NULL;
 	/* if we have an URI, it might contain escaped characters (? : etc)
@@ -1005,13 +986,16 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 	if (new_location == NULL)
 		new_location = no_uri;
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS,
-				    id,
-				    "launcher_location");
-	gconf_client_set_string (client, key, new_location, NULL);
+	settings_instance = panel_layout_get_instance_settings (settings,
+								PANEL_LAUNCHER_SCHEMA);
 
-	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+	g_settings_set_string (settings_instance, PANEL_LOCATION_KEY,
+			       new_location);
 
+	panel_layout_object_create_finish (id);
+
+	g_object_unref (settings_instance);
+	g_object_unref (settings);
 	g_free (no_uri);
 	g_free (id);
 }
@@ -1021,7 +1005,7 @@ panel_launcher_create (PanelToplevel *toplevel,
 		       int            position,
 		       const char    *location)
 {
-	panel_launcher_create_with_id (panel_profile_get_toplevel_id (toplevel),
+	panel_launcher_create_with_id (panel_toplevel_get_id (toplevel),
 				       position,
 				       location);
 }
