@@ -33,14 +33,14 @@
 #include <glib/gi18n.h>
 
 #include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-launch.h>
+#include <libpanel-util/panel-screensaver.h>
 #include <libpanel-util/panel-session-manager.h>
 #include <libpanel-util/panel-show.h>
 
 #include "applet.h"
-#include "panel-config-global.h"
 #include "panel-gconf.h"
-#include "panel-profile.h"
 #include "panel-typebuiltins.h"
 #include "panel-force-quit.h"
 #include "panel-util.h"
@@ -48,16 +48,14 @@
 #include "panel-globals.h"
 #include "panel-run-dialog.h"
 #include "panel-a11y.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
-#include "panel-compatibility.h"
 #include "panel-icon-names.h"
 
 G_DEFINE_TYPE (PanelActionButton, panel_action_button, BUTTON_TYPE_WIDGET)
 
 #define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
-#define LOGOUT_PROMPT_KEY "/apps/gnome-session/options/logout_prompt"
-
 enum {
 	PROP_0,
 	PROP_ACTION_TYPE,
@@ -68,8 +66,6 @@ struct _PanelActionButtonPrivate {
 	PanelActionButtonType  type;
 	AppletInfo            *info;
 
-	guint                  gconf_notify;
-	
 	guint                  dnd_enabled : 1;
 };
 
@@ -82,8 +78,6 @@ static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_FORCE_QUIT,     "force-quit"     },
 	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" },
 	{ PANEL_ACTION_SHUTDOWN,       "shutdown"       },
-	/* compatibility with GNOME < 2.13.90 */
-	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ 0,                           NULL             },
 };
 
@@ -92,48 +86,52 @@ static GConfEnumStringPair panel_action_type_map [] = {
 static void
 panel_action_lock_screen (GtkWidget *widget)
 {
-	panel_lock_screen (gtk_widget_get_screen (widget));
+	panel_screensaver_lock (panel_screensaver_get ());
 }
 
 static gboolean
 screensaver_properties_enabled (void)
 {
-	if (panel_lockdown_get_locked_down () ||
-	    panel_lockdown_get_disable_lock_screen ())
+	char *desktop;
+	gboolean found;
+
+	if (panel_lockdown_get_panels_locked_down_s () ||
+	    panel_lockdown_get_disable_lock_screen_s ())
 		return FALSE;
 
-	return panel_lock_screen_action_available ("prefs");
+	desktop = panel_g_lookup_in_applications_dirs ("gnome-screen-panel.desktop");
+	found = (desktop != NULL);
+	g_free (desktop);
+
+	return found;
 }
 
 static gboolean
-screensaver_enabled (void)
+panel_action_lock_is_enabled (void)
 {
-	if (panel_lockdown_get_disable_lock_screen ())
-		return FALSE;
-
-	return panel_lock_screen_action_available ("lock");
+	return !panel_lockdown_get_disable_lock_screen_s ();
 }
 
 static gboolean
 panel_action_lock_is_disabled (void)
 {
-	return !screensaver_enabled ();
+	return !panel_action_lock_is_enabled ();
 }
 
 static void
 panel_action_lock_setup_menu (PanelActionButton *button)
 {
-	panel_applet_add_callback (button->priv->info,
-				   "activate",
-				   NULL,
-				   _("_Activate Screensaver"),
-				   screensaver_enabled);
-
 	panel_applet_add_callback (button->priv->info,
 				   "lock",
 				   NULL,
 				   _("_Lock Screen"),
-				   screensaver_enabled);
+				   panel_action_lock_is_enabled);
+
+	panel_applet_add_callback (button->priv->info,
+				   "activate",
+				   NULL,
+				   _("_Activate Screensaver"),
+				   NULL);
 
 	panel_applet_add_callback (button->priv->info,
 				   "prefs",
@@ -149,8 +147,16 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 	g_return_if_fail (PANEL_IS_ACTION_BUTTON (button));
 	g_return_if_fail (callback_name != NULL);
 
-	panel_lock_screen_action (gtk_widget_get_screen (GTK_WIDGET (button)),
-				  callback_name);
+	if (g_strcmp0 (callback_name, "lock") == 0)
+		panel_screensaver_lock (panel_screensaver_get ());
+	else if (g_strcmp0 (callback_name, "activate") == 0)
+		panel_screensaver_activate (panel_screensaver_get ());
+	else if (g_strcmp0 (callback_name, "prefs") == 0)
+		panel_launch_desktop_file ("gnome-screen-panel.desktop",
+					   gtk_widget_get_screen (GTK_WIDGET (button)),
+					   NULL);
+	else
+		g_assert_not_reached ();
 }
 
 /* Log Out
@@ -158,26 +164,11 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 static void
 panel_action_logout (GtkWidget *widget)
 {
-	PanelSessionManager *manager;
-	gboolean             not_prompt;
-
-	not_prompt = gconf_client_get_bool (panel_gconf_get_client (),
-					    LOGOUT_PROMPT_KEY, NULL);
-	/* this avoids handling errors from gconf since prompting is
-	 * safer */
-	not_prompt = !not_prompt;
-
-	manager = panel_session_manager_get ();
-
-	if (not_prompt)
-		panel_session_manager_request_logout (manager,
-						      PANEL_SESSION_MANAGER_LOGOUT_MODE_NO_CONFIRMATION);
-	else
-		/* FIXME: we need to use widget to get the screen for the
-		 * confirmation dialog, see
-		 * http://bugzilla.gnome.org/show_bug.cgi?id=536914 */
-		panel_session_manager_request_logout (manager,
-						      PANEL_SESSION_MANAGER_LOGOUT_MODE_NORMAL);
+	/* FIXME: we need to use widget to get the screen for the
+	 * confirmation dialog, see
+	 * http://bugzilla.gnome.org/show_bug.cgi?id=536914 */
+	panel_session_manager_request_logout (panel_session_manager_get (),
+					      PANEL_SESSION_MANAGER_LOGOUT_MODE_NORMAL);
 }
 
 static void
@@ -194,7 +185,7 @@ panel_action_shutdown_reboot_is_disabled (void)
 {
 	PanelSessionManager *manager;
 
-	if (panel_lockdown_get_disable_log_out())
+	if (panel_lockdown_get_disable_log_out_s ())
 		return TRUE;
 
 	manager = panel_session_manager_get ();
@@ -280,7 +271,6 @@ typedef struct {
 	char                   *icon_name;
 	char                   *text;
 	char                   *tooltip;
-	char                   *help_index;
 	char                   *drag_id;
 	void                  (*invoke)      (GtkWidget         *widget);
 	void                  (*setup_menu)  (PanelActionButton *button);
@@ -294,7 +284,7 @@ typedef struct {
 static PanelAction actions [] = {
 	{
 		PANEL_ACTION_NONE,
-		NULL, NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL
 	},
 	{
@@ -302,7 +292,6 @@ static PanelAction actions [] = {
 		PANEL_ICON_LOCKSCREEN,
 		N_("Lock Screen"),
 		N_("Protect your computer from unauthorized use"),
-		"gospanel-21",
 		"ACTION:lock:NEW",
 		panel_action_lock_screen,
 		panel_action_lock_setup_menu,
@@ -317,27 +306,24 @@ static PanelAction actions [] = {
 		 * "1" (msgctxt: "panel:showusername")) */
 		N_("Log Out..."),
 		N_("Log out of this session to log in as a different user"),
-		"gospanel-20",
 		"ACTION:logout:NEW",
 		panel_action_logout, NULL, NULL,
-		panel_lockdown_get_disable_log_out
+		panel_lockdown_get_disable_log_out_s
 	},
 	{
 		PANEL_ACTION_RUN,
 		PANEL_ICON_RUN,
 		N_("Run Application..."),
 		N_("Run an application by typing a command or choosing from a list"),
-		"gospanel-555",
 		"ACTION:run:NEW",
 		panel_action_run_program, NULL, NULL,
-		panel_lockdown_get_disable_command_line
+		panel_lockdown_get_disable_command_line_s
 	},
 	{
 		PANEL_ACTION_SEARCH,
 		PANEL_ICON_SEARCHTOOL,
 		N_("Search for Files..."),
 		N_("Locate documents and folders on this computer by name or content"),
-		"gospanel-554",
 		"ACTION:search:NEW",
 		panel_action_search, NULL, NULL, NULL
 	},
@@ -346,17 +332,15 @@ static PanelAction actions [] = {
 		PANEL_ICON_FORCE_QUIT,
 		N_("Force Quit"),
 		N_("Force a misbehaving application to quit"),
-		"gospanel-563",
 		"ACTION:force-quit:NEW",
 		panel_action_force_quit, NULL, NULL,
-		panel_lockdown_get_disable_force_quit
+		panel_lockdown_get_disable_force_quit_s
 	},
 	{
 		PANEL_ACTION_CONNECT_SERVER,
 		PANEL_ICON_REMOTE, //FIXME icon
 		N_("Connect to Server..."),
 		N_("Connect to a remote computer or shared disk"),
-		"nautilus-server-connect",
 		"ACTION:connect-server:NEW",
 		panel_action_connect_server, NULL, NULL, NULL
 	},
@@ -365,16 +349,9 @@ static PanelAction actions [] = {
 		PANEL_ICON_SHUTDOWN,
 		N_("Shut Down..."),
 		N_("Shut down the computer"),
-		"gospanel-20",
 		"ACTION:shutdown:NEW",
 		panel_action_shutdown, NULL, NULL,
 		panel_action_shutdown_reboot_is_disabled
-	},
-	/* deprecated actions */
-	{
-		PANEL_ACTION_SCREENSHOT,
-		NULL, NULL, NULL, NULL, NULL,
-		NULL, NULL, NULL, NULL
 	}
 };
 
@@ -383,7 +360,7 @@ panel_action_get_is_deprecated (PanelActionButtonType type)
 {
 	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
 
-	return (type >= PANEL_ACTION_SCREENSHOT);
+	return (type > PANEL_ACTION_LAST_NON_DEPRECATED);
 }
 
 gboolean
@@ -443,8 +420,11 @@ panel_action_get_drag_id (PanelActionButtonType type)
 }
 
 static void
-panel_action_button_update_sensitivity (PanelActionButton *button)
+panel_action_button_update_sensitivity (PanelLockdown *lockdown,
+					gpointer       user_data)
 {
+	PanelActionButton *button = user_data;
+
 	if (actions [button->priv->type].is_disabled)
 		button_widget_set_activatable (BUTTON_WIDGET (button),
 					       !actions [button->priv->type].is_disabled ());
@@ -458,13 +438,6 @@ panel_action_button_finalize (GObject *object)
 	button->priv->info = NULL;
 	button->priv->type = PANEL_ACTION_NONE;
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_action_button_update_sensitivity),
-				      button);
-
-	gconf_client_notify_remove (panel_gconf_get_client (),
-				    button->priv->gconf_notify);
-	button->priv->gconf_notify = 0;
-
 	G_OBJECT_CLASS (panel_action_button_parent_class)->finalize (object);
 }
 
@@ -557,15 +530,6 @@ panel_action_button_clicked (GtkButton *gtk_button)
 	g_return_if_fail (button->priv->type > PANEL_ACTION_NONE);
 	g_return_if_fail (button->priv->type < PANEL_ACTION_LAST);
 
-	if (panel_global_config_get_drawer_auto_close ()) {
-		PanelToplevel *toplevel;
-
-		toplevel = PANEL_WIDGET (gtk_widget_get_parent (GTK_WIDGET (button)))->toplevel;
-
-		if (panel_toplevel_get_is_attached (toplevel))
-			panel_toplevel_hide (toplevel, FALSE, -1);
-	}
-
 	if (actions [button->priv->type].invoke)
 		actions [button->priv->type].invoke (GTK_WIDGET (button));
 }
@@ -595,7 +559,7 @@ panel_action_button_class_init (PanelActionButtonClass *klass)
 					   "The type of action this button implements",
 					   PANEL_TYPE_ACTION_BUTTON_TYPE,
 					   PANEL_ORIENTATION_TOP,
-					   G_PARAM_READWRITE));
+					   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_object_class_install_property (
 			gobject_class,
@@ -615,7 +579,6 @@ panel_action_button_init (PanelActionButton *button)
 	button->priv->type = PANEL_ACTION_NONE;
 	button->priv->info = NULL;
 
-	button->priv->gconf_notify = 0;
 	button->priv->dnd_enabled  = FALSE;
 }
 
@@ -640,46 +603,7 @@ panel_action_button_set_type (PanelActionButton     *button,
 				     _(actions [type].tooltip));
 	panel_a11y_set_atk_name_desc (GTK_WIDGET (button), _(actions [type].tooltip), NULL);
 
-	panel_action_button_update_sensitivity (button);
-}
-
-static void
-panel_action_button_type_changed (GConfClient       *client,
-				  guint              cnxn_id,
-				  GConfEntry        *entry,
-				  PanelActionButton *button)
-{
-	int         type;
-	const char *action_type;
-
-	g_return_if_fail (PANEL_IS_ACTION_BUTTON (button));
-
-	if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
-		return;
-
-	action_type = gconf_value_get_string (entry->value);
-
-	if (!gconf_string_to_enum (panel_action_type_map, action_type, &type))
-		return;
-
-	panel_action_button_set_type (button, type);
-}
-
-static void
-panel_action_button_connect_to_gconf (PanelActionButton *button)
-{
-	const char *key;
-
-	key = panel_gconf_full_key (
-			PANEL_GCONF_OBJECTS, button->priv->info->id, "action_type");
-
-	button->priv->gconf_notify =
-		gconf_client_notify_add (panel_gconf_get_client (), key, 
-					 (GConfClientNotifyFunc) panel_action_button_type_changed,
-					 button, NULL, NULL);
-
-	panel_lockdown_notify_add (G_CALLBACK (panel_action_button_update_sensitivity),
-				   button);
+	panel_action_button_update_sensitivity (panel_lockdown_get (), button);
 }
 
 static void
@@ -690,51 +614,37 @@ panel_action_button_style_updated (PanelActionButton *button)
 }
 
 static void
-panel_action_button_load (PanelActionButtonType  type,
-			  PanelWidget           *panel,
-			  gboolean               locked,
-			  int                    position,
-			  gboolean               exactpos,
-			  const char            *id,
-			  gboolean               compatibility)
+panel_action_button_load_helper (PanelWidget           *panel,
+				 const char            *id,
+				 GSettings             *settings,
+				 PanelActionButtonType  type)
 {
 	PanelActionButton *button;
-	PanelObjectType    object_type;
 
 	g_return_if_fail (panel != NULL);
 
 	button = g_object_new (PANEL_TYPE_ACTION_BUTTON, "action-type", type, NULL);
 
-	object_type = PANEL_OBJECT_ACTION;
-	if (compatibility) { /* Backward compatibility with GNOME 2.0.x */
-		if (type == PANEL_ACTION_LOCK)
-			object_type = PANEL_OBJECT_LOCK;
-		else if (type == PANEL_ACTION_LOGOUT)
-			object_type = PANEL_OBJECT_LOGOUT;
-	}
-
-	button->priv->info = panel_applet_register (GTK_WIDGET (button),
-						    NULL, NULL,
-						    panel, locked, position,
-						    exactpos, object_type, id);
+	button->priv->info = panel_applet_register (GTK_WIDGET (button), panel,
+						    PANEL_OBJECT_ACTION, id,
+						    settings,
+						    NULL, NULL);
 	if (!button->priv->info) {
 		gtk_widget_destroy (GTK_WIDGET (button));
 		return;
 	}
 
-	panel_applet_add_callback (button->priv->info,
-				   "help",
-				   GTK_STOCK_HELP,
-				   _("_Help"),
-				   NULL);
-
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
 
 	if (actions [button->priv->type].setup_menu)
 		actions [button->priv->type].setup_menu (button);
 
-	panel_action_button_connect_to_gconf (button);
+	panel_lockdown_on_notify (panel_lockdown_get (),
+				  NULL,
+				  G_OBJECT (button),
+				  panel_action_button_update_sensitivity,
+				  button);
 
 	g_signal_connect (button, "style-updated",
 			  G_CALLBACK (panel_action_button_style_updated), NULL);
@@ -745,76 +655,39 @@ panel_action_button_create (PanelToplevel         *toplevel,
 			    int                    position,
 			    PanelActionButtonType  type)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *id;
+	const char *detail;
 
-	client  = panel_gconf_get_client ();
+	detail = gconf_enum_to_string (panel_action_type_map, type);
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_ACTION, toplevel, position, FALSE);
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "action_type");
-	gconf_client_set_string (client,
-				 key,
-				 gconf_enum_to_string (panel_action_type_map, type),
-				 NULL);
-
-	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
-
-	g_free (id);
-}
-
-/* This is only for backwards compatibility with 2.0.x
- * We load an old-style lock/logout button as an action
- * button but make sure to retain the lock/logout configuration
- * so logging back into 2.0.x still works.
- */
-void
-panel_action_button_load_compatible (PanelObjectType  object_type,
-				     PanelWidget     *panel,
-				     gboolean         locked,
-				     int              position,
-				     gboolean         exactpos,
-				     const char      *id)
-{
-	PanelActionButtonType action_type;
-
-	g_assert (object_type == PANEL_OBJECT_LOGOUT || object_type == PANEL_OBJECT_LOCK);
-
-	action_type = object_type == PANEL_OBJECT_LOGOUT ? PANEL_ACTION_LOGOUT : PANEL_ACTION_LOCK;
-
-	panel_action_button_load (action_type, panel, locked, position, exactpos, id, TRUE);
+	panel_layout_object_create (PANEL_OBJECT_ACTION,
+				    detail,
+				    panel_toplevel_get_id (toplevel),
+				    position, FALSE);
 }
 
 void
-panel_action_button_load_from_gconf (PanelWidget *panel,
-				     gboolean     locked,
-				     int          position,
-				     gboolean     exactpos,
-				     const char  *id)
+panel_action_button_load (PanelWidget *panel,
+			  const char  *id,
+			  GSettings *settings,
+			  const char  *detail_for_type)
 {
-	int          type;
-	const char  *key;
-	char        *action_type;
+	int type = PANEL_ACTION_NONE;
+	int i;
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "action_type");
-	action_type = gconf_client_get_string (panel_gconf_get_client (), key, NULL);
+	for (i = 0; panel_action_type_map[i].str != NULL; i++) {
+		if (g_strcmp0 (detail_for_type,
+			       panel_action_type_map[i].str) == 0) {
+			type = panel_action_type_map[i].enum_value;
+			break;
+		}
+	}
 
-	if (!gconf_string_to_enum (panel_action_type_map, action_type, &type)) {
-		g_warning ("Unkown action type '%s' from %s", action_type, key);
-		g_free (action_type);
+	if (type == PANEL_ACTION_NONE) {
+		g_warning ("Unkown action type '%s", detail_for_type);
 		return;
 	}
 
-	g_free (action_type);
-
-	/* compatibility: migrate from GNOME < 2.13.90 */
-	if (type == PANEL_ACTION_SCREENSHOT)
-		panel_compatibility_migrate_screenshot_action (panel_gconf_get_client (),
-							       id);
-	else
-		panel_action_button_load (type, panel, locked,
-					  position, exactpos, id, FALSE);
+	panel_action_button_load_helper (panel, id, settings, type);
 }
 
 void
@@ -826,20 +699,6 @@ panel_action_button_invoke_menu (PanelActionButton *button,
 	g_return_if_fail (button->priv->type > PANEL_ACTION_NONE &&
 			  button->priv->type < PANEL_ACTION_LAST);
 
-	if (!strcmp (callback_name, "help")) {
-		GdkScreen *screen;
-
-		if (!actions [button->priv->type].help_index)
-			return;
-
-		screen = gtk_widget_get_screen (GTK_WIDGET (button));
-
-		panel_show_help (screen, "user-guide",
-				 actions [button->priv->type].help_index, NULL);
-
-		return;
-	}
-
 	if (actions [button->priv->type].invoke_menu)
 		actions [button->priv->type].invoke_menu (button, callback_name);
 }
