@@ -32,7 +32,10 @@
 #include <cairo.h>
 #include <cairo-xlib.h>
 
+#include <libpanel-util/panel-glib.h>
+
 #include "panel-background-monitor.h"
+#include "panel-schemas.h"
 #include "panel-util.h"
 
 
@@ -331,7 +334,26 @@ get_scaled_and_rotated_pixbuf (PanelBackground *background)
 	width  = orig_width;
 	height = orig_height;
 
-	if (background->fit_image) {
+	switch (background->style_image) {
+	case PANEL_BACKGROUND_IMAGE_STYLE_NONE:
+		if (background->orientation == GTK_ORIENTATION_VERTICAL &&
+			   background->rotate_image) {
+			int tmp = width;
+			width = height;
+			height = tmp;
+		}
+		break;
+	case PANEL_BACKGROUND_IMAGE_STYLE_STRETCH:
+		if (background->orientation == GTK_ORIENTATION_VERTICAL &&
+		    background->rotate_image) {
+			width  = panel_height;
+			height = panel_width;
+		} else {
+			width  = panel_width;
+			height = panel_height;
+		}
+		break;
+	case PANEL_BACKGROUND_IMAGE_STYLE_FIT:
 		switch (background->orientation) {
 		case GTK_ORIENTATION_HORIZONTAL:
 			width  = orig_width * panel_height / orig_height;
@@ -350,20 +372,10 @@ get_scaled_and_rotated_pixbuf (PanelBackground *background)
 			g_assert_not_reached ();
 			break;
 		}
-	} else if (background->stretch_image) {
-		if (background->orientation == GTK_ORIENTATION_VERTICAL &&
-		    background->rotate_image) {
-			width  = panel_height;
-			height = panel_width;
-		} else {
-			width  = panel_width;
-			height = panel_height;
-		}
-	} else if (background->orientation == GTK_ORIENTATION_VERTICAL &&
-		   background->rotate_image) {
-		int tmp = width;
-		width = height;
-		height = tmp;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
 	}
 
 	if (width == orig_width &&
@@ -506,7 +518,7 @@ load_background_file (PanelBackground *background)
 	panel_background_update_has_alpha (background);
 }
 
-void
+static void
 panel_background_set_type (PanelBackground     *background,
 			   PanelBackgroundType  type)
 {
@@ -535,7 +547,7 @@ panel_background_set_color_no_update (PanelBackground *background,
         panel_background_update_has_alpha (background);
 }
 
-void
+static void
 panel_background_set_color (PanelBackground *background,
                             const GdkRGBA   *color)
 {
@@ -550,8 +562,8 @@ panel_background_set_color (PanelBackground *background,
 }
 
 static void
-panel_background_set_image_no_update (PanelBackground *background,
-				      const char      *image)
+panel_background_set_image_uri_no_update (PanelBackground *background,
+					  const char      *uri)
 {
 	if (background->loaded_image)
 		g_object_unref (background->loaded_image);
@@ -561,66 +573,62 @@ panel_background_set_image_no_update (PanelBackground *background,
 		g_free (background->image);
 	background->image = NULL;
 
-	if (image && image [0])
-		background->image = g_strdup (image);
+	if (!PANEL_GLIB_STR_EMPTY (uri)) {
+		GFile *file;
+		file = g_file_new_for_uri (uri);
+
+		if (g_file_is_native (file))
+			background->image = g_file_get_path (file);
+
+		g_object_unref (file);
+	}
 
 	panel_background_update_has_alpha (background);
 }
 
-void
-panel_background_set_image (PanelBackground *background,
-			    const char      *image)
+static void
+panel_background_set_image_uri (PanelBackground *background,
+				const char      *uri)
 {
-	if (!background->image && !image)
+	GFile *file;
+	char  *current_uri = NULL;
+
+	if (!background->image && !uri)
 		return;
 
-	if (background->image && image && !strcmp (background->image, image))
+	if (background->image) {
+		file = g_file_new_for_path (background->image);
+		current_uri = g_file_get_uri (file);
+		g_object_unref (file);
+	}
+
+	if (g_strcmp0 (current_uri, uri) == 0) {
+		g_free (current_uri);
 		return;
+	}
+	g_free (current_uri);
 
 	free_transformed_resources (background);
-	panel_background_set_image_no_update (background, image);
+	panel_background_set_image_uri_no_update (background, uri);
 	panel_background_transform (background);
 }
 
 static void
-panel_background_set_fit_no_update (PanelBackground *background,
-				    gboolean         fit_image)
+panel_background_set_image_style_no_update (PanelBackground           *background,
+					    PanelBackgroundImageStyle  style)
 {
-	background->fit_image = fit_image != FALSE;
-}
-
-void
-panel_background_set_fit (PanelBackground *background,
-			  gboolean         fit_image)
-{
-	fit_image = fit_image != FALSE;
-
-	if (background->fit_image == fit_image)
-		return;
-
-	free_transformed_resources (background);
-	panel_background_set_fit_no_update (background, fit_image);
-	panel_background_transform (background);
+	background->style_image = style;
 }
 
 static void
-panel_background_set_stretch_no_update (PanelBackground *background,
-				        gboolean         stretch_image)
+panel_background_set_image_style (PanelBackground           *background,
+				  PanelBackgroundImageStyle  style)
 {
-	background->stretch_image = stretch_image != FALSE;
-}
-
-void
-panel_background_set_stretch (PanelBackground *background,
-			      gboolean         stretch_image)
-{
-	stretch_image = stretch_image != FALSE;
-
-	if (background->stretch_image == stretch_image)
+	if (background->style_image == style)
 		return;
 
 	free_transformed_resources (background);
-	panel_background_set_stretch_no_update (background, stretch_image);
+	panel_background_set_image_style_no_update (background, style);
 	panel_background_transform (background);
 }
 
@@ -631,7 +639,7 @@ panel_background_set_rotate_no_update (PanelBackground *background,
 	background->rotate_image = rotate_image != FALSE;
 }
 
-void
+static void
 panel_background_set_rotate (PanelBackground *background,
 			     gboolean         rotate_image)
 {
@@ -645,20 +653,80 @@ panel_background_set_rotate (PanelBackground *background,
 	panel_background_transform (background);
 }
 
-void
-panel_background_set (PanelBackground     *background,
-		      PanelBackgroundType  type,
-		      const GdkRGBA       *color,
-		      const char          *image,
-		      gboolean             fit_image,
-		      gboolean             stretch_image,
-		      gboolean             rotate_image)
+static void
+panel_background_settings_changed (GSettings       *settings,
+				   char            *key,
+				   PanelBackground *background)
 {
-	panel_background_set_color_no_update (background, color);
-	panel_background_set_image_no_update (background, image);
-	panel_background_set_fit_no_update (background, fit_image);
-	panel_background_set_stretch_no_update (background, stretch_image);
+	char     *value_str;
+	gboolean  value_boolean;
+
+	if (g_strcmp0 (key, PANEL_BACKGROUND_TYPE_KEY) == 0) {
+		PanelBackgroundType type;
+		type = g_settings_get_enum (settings, key);
+		panel_background_set_type (background, type);
+	} else if (g_strcmp0 (key, PANEL_BACKGROUND_COLOR_KEY) == 0) {
+		GdkRGBA color;
+		value_str = g_settings_get_string (settings, key);
+		if (gdk_rgba_parse (&color, value_str))
+			panel_background_set_color (background, &color);
+		g_free (value_str);
+	} else if (g_strcmp0 (key, PANEL_BACKGROUND_IMAGE_URI_KEY) == 0) {
+		value_str = g_settings_get_string (settings, key);
+		panel_background_set_image_uri (background, value_str);
+		g_free (value_str);
+	} else if (g_strcmp0 (key, PANEL_BACKGROUND_IMAGE_STYLE_KEY) == 0) {
+		PanelBackgroundImageStyle style;
+		style = g_settings_get_enum (settings, key);
+		panel_background_set_image_style (background, style);
+	} else if (g_strcmp0 (key, PANEL_BACKGROUND_IMAGE_ROTATE_KEY) == 0) {
+		value_boolean = g_settings_get_boolean (settings, key);
+		panel_background_set_rotate (background, value_boolean);
+	} else {
+		g_assert_not_reached ();
+	}
+}
+
+void
+panel_background_settings_init (PanelBackground *background,
+				GSettings       *settings)
+{
+	PanelBackgroundType  type;
+	char                *color_str;
+	GdkRGBA              color;
+	char                *image;
+	PanelBackgroundImageStyle style_image;
+	gboolean             rotate_image;
+
+	g_assert (background->settings == NULL);
+
+	background->settings = g_object_ref (settings);
+	g_signal_connect (background->settings, "changed",
+			  G_CALLBACK (panel_background_settings_changed),
+			  background);
+
+	color_str = g_settings_get_string (background->settings,
+					   PANEL_BACKGROUND_COLOR_KEY);
+	if (!gdk_rgba_parse (&color, color_str))
+		gdk_rgba_parse (&color, PANEL_BACKGROUND_COLOR_DEFAULT);
+	panel_background_set_color_no_update (background, &color);
+	g_free (color_str);
+
+	image = g_settings_get_string (background->settings,
+				       PANEL_BACKGROUND_IMAGE_URI_KEY);
+	panel_background_set_image_uri_no_update (background, image);
+	g_free (image);
+
+	style_image = g_settings_get_enum (background->settings,
+					   PANEL_BACKGROUND_IMAGE_STYLE_KEY);
+	panel_background_set_image_style_no_update (background, style_image);
+
+	rotate_image = g_settings_get_boolean (background->settings,
+					       PANEL_BACKGROUND_IMAGE_ROTATE_KEY);
 	panel_background_set_rotate_no_update (background, rotate_image);
+
+	type = g_settings_get_enum (background->settings,
+				    PANEL_BACKGROUND_TYPE_KEY);
 	panel_background_set_type (background, type);
 }
 
@@ -734,8 +802,7 @@ panel_background_change_region (PanelBackground *background,
 			need_to_retransform = TRUE;
 		} else if ((background->region.width != width ||
 			    background->region.height != height) &&
-			   (background->fit_image ||
-			    background->stretch_image)) {
+			   (background->style_image != PANEL_BACKGROUND_IMAGE_STYLE_NONE)) {
 			/* or if the size changes and we are 
 			   stretching or fitting the image */
 			need_to_retransform = TRUE;
@@ -778,6 +845,8 @@ panel_background_init (PanelBackground              *background,
 		       PanelBackgroundChangedNotify  notify_changed,
 		       gpointer                      user_data)
 {
+	background->settings = NULL;
+
 	background->type = PANEL_BACK_NONE;
 	background->notify_changed = notify_changed;
 	background->user_data = user_data;
@@ -810,8 +879,7 @@ panel_background_init (PanelBackground              *background,
 	background->default_color.blue  = 0.;
 	background->default_color.alpha = 1.;
 
-	background->fit_image     = FALSE;
-	background->stretch_image = FALSE;
+	background->style_image   = PANEL_BACKGROUND_IMAGE_STYLE_NONE;
 	background->rotate_image  = FALSE;
 
 	background->has_alpha = FALSE;
@@ -827,6 +895,10 @@ panel_background_free (PanelBackground *background)
 
 	free_transformed_resources (background);
 
+	if (background->settings)
+		g_object_unref (background->settings);
+	background->settings = NULL;
+
 	if (background->image)
 		g_free (background->image);
 	background->image = NULL;
