@@ -42,7 +42,7 @@
 #include "menu.h"
 #include "panel-menu-items.h"
 #include "panel-globals.h"
-#include "panel-profile.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
 #include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
@@ -179,14 +179,17 @@ panel_menu_bar_init (PanelMenuBar *menubar)
 				   menubar->priv->applications_menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->applications_item);
+	gtk_widget_show (menubar->priv->applications_item);
 
 	menubar->priv->places_item = panel_place_menu_item_new (FALSE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->places_item);
+	gtk_widget_show (menubar->priv->places_item);
 
 	menubar->priv->desktop_item = panel_desktop_menu_item_new (FALSE, TRUE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->desktop_item);
+	gtk_widget_show (menubar->priv->desktop_item);
 
 	panel_menu_bar_setup_tooltip (menubar);
 
@@ -380,12 +383,10 @@ panel_menu_bar_on_draw (GtkWidget *widget,
 	return FALSE;
 }
 
-static void
+void
 panel_menu_bar_load (PanelWidget *panel,
-		     gboolean     locked,
-		     int          position,
-		     gboolean     exactpos,
-		     const char  *id)
+		     const char  *id,
+		     GSettings   *settings)
 {
 	PanelMenuBar *menubar;
 
@@ -394,9 +395,10 @@ panel_menu_bar_load (PanelWidget *panel,
 	menubar = g_object_new (PANEL_TYPE_MENU_BAR, NULL);
 
 	menubar->priv->info = panel_applet_register (
-					GTK_WIDGET (menubar), NULL, NULL,
-					panel, locked, position, exactpos,
-					PANEL_OBJECT_MENU_BAR, id);
+					GTK_WIDGET (menubar), panel,
+					PANEL_OBJECT_MENU_BAR, id,
+					settings,
+					NULL, NULL);
 	if (!menubar->priv->info) {
 		gtk_widget_destroy (GTK_WIDGET (menubar));
 		return;
@@ -408,7 +410,7 @@ panel_menu_bar_load (PanelWidget *panel,
 					   "edit",
 					   NULL,
 					   _("_Edit Menus"),
-					   panel_lockdown_get_not_locked_down);
+					   panel_lockdown_get_not_panels_locked_down_s);
 	}
 
 	panel_applet_add_callback (menubar->priv->info,
@@ -428,25 +430,13 @@ panel_menu_bar_load (PanelWidget *panel,
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
 }
 
-void
-panel_menu_bar_load_from_gconf (PanelWidget *panel,
-				gboolean     locked,
-				int          position,
-				gboolean     exactpos,
-				const char  *id)
-{
-	panel_menu_bar_load (panel, locked, position, exactpos, id);
-}
-
 void
 panel_menu_bar_create (PanelToplevel *toplevel,
 		       int            position)
 {
-	char *id;
-
-	id = panel_profile_prepare_object (PANEL_OBJECT_MENU_BAR, toplevel, position, FALSE);
-	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
-	g_free (id);
+	panel_layout_object_create (PANEL_OBJECT_MENU_BAR, NULL,
+				    panel_toplevel_get_id (toplevel),
+				    position, FALSE);
 }
 
 void
