@@ -32,19 +32,20 @@
 #include <gmenu-tree.h>
 
 #include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-launch.h>
 #include <libpanel-util/panel-show.h>
 
 #include "applet.h"
-#include "panel-compatibility.h"
 #include "panel-widget.h"
 #include "panel-util.h"
-#include "panel-profile.h"
 #include "panel-globals.h"
 #include "menu.h"
 #include "panel-lockdown.h"
 #include "panel-a11y.h"
+#include "panel-layout.h"
 #include "panel-icon-names.h"
+#include "panel-schemas.h"
 
 G_DEFINE_TYPE (PanelMenuButton, panel_menu_button, BUTTON_TYPE_WIDGET)
 
@@ -55,8 +56,6 @@ enum {
 	PROP_MENU_PATH,
 	PROP_CUSTOM_ICON,
 	PROP_TOOLTIP,
-	PROP_USE_MENU_PATH,
-	PROP_USE_CUSTOM_ICON,
 	PROP_DND_ENABLED
 };
 
@@ -64,7 +63,6 @@ typedef enum {
 	FIRST_MENU,
 	APPLICATIONS_MENU,
 #define DEFAULT_MENU      APPLICATIONS_MENU
-	SETTINGS_MENU,
 	GNOMECC_MENU,
 	LAST_MENU
 } MenuPathRoot;
@@ -77,15 +75,12 @@ typedef struct {
 
 static MenuPathRootItem root_items [] = {
 	{ APPLICATIONS_MENU, "applications", "applications.menu" },
-	{ GNOMECC_MENU,      "gnomecc",      "gnomecc.menu"      },
-	/* compatibility with GNOME < 2.91.0 */
-	{ SETTINGS_MENU,     "settings",     NULL                }
+	{ GNOMECC_MENU,      "gnomecc",      "gnomecc.menu"      }
 };
 
 struct _PanelMenuButtonPrivate {
 	PanelToplevel         *toplevel;
-	guint                  gconf_notify;
-	char                  *applet_id;
+	GSettings             *settings_instance;
 
 	GtkWidget             *menu;
 
@@ -94,13 +89,9 @@ struct _PanelMenuButtonPrivate {
 	char                  *tooltip;
 
 	MenuPathRoot           path_root;
-	guint                  use_menu_path : 1;
-	guint                  use_custom_icon : 1;
 	guint                  dnd_enabled : 1;
 };
 
-static void panel_menu_button_disconnect_from_gconf (PanelMenuButton *button);
-static void panel_menu_button_recreate_menu         (PanelMenuButton *button);
 static void panel_menu_button_set_icon              (PanelMenuButton *button);
 
 static AtkObject *panel_menu_button_get_accessible  (GtkWidget       *widget);
@@ -174,17 +165,14 @@ panel_menu_button_init (PanelMenuButton *button)
 {
 	button->priv = PANEL_MENU_BUTTON_GET_PRIVATE (button);
 
-	button->priv->applet_id    = NULL;
-	button->priv->toplevel     = NULL;
-	button->priv->gconf_notify = 0;
+	button->priv->toplevel          = NULL;
+	button->priv->settings_instance = NULL;
 
 	button->priv->menu_path   = NULL;
 	button->priv->custom_icon = NULL;
 	button->priv->tooltip = NULL;
 
 	button->priv->path_root       = LAST_MENU;
-	button->priv->use_menu_path   = FALSE;
-	button->priv->use_custom_icon = FALSE;
 }
 
 static void
@@ -192,19 +180,15 @@ panel_menu_button_finalize (GObject *object)
 {
 	PanelMenuButton *button = PANEL_MENU_BUTTON (object);
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_button_recreate_menu),
-				      button);
-
-	panel_menu_button_disconnect_from_gconf (button);
-
 	if (button->priv->menu) {
 		/* detaching the menu will kill our reference */
 		gtk_menu_detach (GTK_MENU (button->priv->menu));
 		button->priv->menu = NULL;
 	}
 
-	g_free (button->priv->applet_id);
-	button->priv->applet_id = NULL;
+	if (button->priv->settings_instance)
+		g_object_unref (button->priv->settings_instance);
+	button->priv->settings_instance = NULL;
 
 	g_free (button->priv->menu_path);
 	button->priv->menu_path = NULL;
@@ -240,12 +224,6 @@ panel_menu_button_get_property (GObject    *object,
 	case PROP_TOOLTIP:
 		g_value_set_string (value, button->priv->tooltip);
 		break;
-	case PROP_USE_MENU_PATH:
-		g_value_set_boolean (value, button->priv->use_menu_path);
-		break;
-	case PROP_USE_CUSTOM_ICON:
-		g_value_set_boolean (value, button->priv->use_custom_icon);
-		break;
 	case PROP_DND_ENABLED:
 		g_value_set_boolean (value, button->priv->dnd_enabled);
 		break;
@@ -277,12 +255,6 @@ panel_menu_button_set_property (GObject      *object,
 	case PROP_TOOLTIP:
 		panel_menu_button_set_tooltip (button, g_value_get_string (value));
 		break;
-	case PROP_USE_MENU_PATH:
-		panel_menu_button_set_use_menu_path (button, g_value_get_boolean (value));
-		break;
-	case PROP_USE_CUSTOM_ICON:
-		panel_menu_button_set_use_custom_icon (button, g_value_get_boolean (value));
-		break;
 	case PROP_DND_ENABLED:
 		panel_menu_button_set_dnd_enabled (button, g_value_get_boolean (value));
 		break;
@@ -386,8 +358,8 @@ panel_menu_button_create_menu (PanelMenuButton *button)
 
 	panel_widget = panel_toplevel_get_panel_widget (button->priv->toplevel);
 
-	if (button->priv->use_menu_path          &&
-	    button->priv->path_root > FIRST_MENU &&
+	if (!PANEL_GLIB_STR_EMPTY (button->priv->menu_path) &&
+	    button->priv->path_root > FIRST_MENU            &&
 	    button->priv->path_root < LAST_MENU) {
 		const char *filename;
 
@@ -411,14 +383,6 @@ panel_menu_button_create_menu (PanelMenuButton *button)
 	return button->priv->menu;
 }
 
-static void
-panel_menu_button_recreate_menu (PanelMenuButton *button)
-{
-	if (button->priv->menu)
-		gtk_widget_destroy (button->priv->menu);
-	button->priv->menu = NULL;
-}
-
 void
 panel_menu_button_popup_menu (PanelMenuButton *button,
 			      guint            n_button,
@@ -531,24 +495,6 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass)
 					     NULL,
 					     G_PARAM_READWRITE));
 
-	g_object_class_install_property (
-			gobject_class,
-			PROP_USE_MENU_PATH,
-                        g_param_spec_boolean ("use-menu-path",
-					      "Use Menu Path",
-					      "Use the path specified by the menu-path property",
-					      FALSE,
-					      G_PARAM_READWRITE));
-
-	g_object_class_install_property (
-			gobject_class,
-			PROP_USE_CUSTOM_ICON,
-                        g_param_spec_boolean ("use-custom-icon",
-					      "Use Custom Icon",
-					      "Use the icon specified by the custom-icon property",
-					      FALSE,
-					      G_PARAM_READWRITE));
-
 	g_object_class_install_property (
 			gobject_class,
 			PROP_DND_ENABLED,
@@ -560,89 +506,33 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass)
 }
 
 static void
-panel_menu_button_gconf_notify (GConfClient     *client,
-				guint            cnxn_id,
-				GConfEntry      *entry,
-				PanelMenuButton *button)
+panel_menu_button_settings_changed (GSettings       *settings,
+				    char            *key,
+				    PanelMenuButton *button)
 {
-	GConfValue *value;
-	const char *key;
+	char *value = NULL;
 
-	key = panel_gconf_basename (gconf_entry_get_key (entry));
-
-	value = entry->value;
-
-	if (!strcmp (key, "menu_path")) {
-		if (value && value->type == GCONF_VALUE_STRING)
-			panel_menu_button_set_menu_path (button,
-							 gconf_value_get_string (value));
-	} else if (!strcmp (key, "custom_icon")) {
-		if (value && value->type == GCONF_VALUE_STRING)
-			panel_menu_button_set_custom_icon (button,
-							   gconf_value_get_string (value));
-	} else if (!strcmp (key, "tooltip")) {
-		if (value && value->type == GCONF_VALUE_STRING)
-			panel_menu_button_set_tooltip (button,
-						       gconf_value_get_string (value));
-	} else if (!strcmp (key, "use_menu_path")) {
-		if (value && value->type == GCONF_VALUE_BOOL)
-			panel_menu_button_set_use_menu_path (button,
-							     gconf_value_get_bool (value));
-	} else if (!strcmp (key, "use_custom_icon")) {
-		if (value && value->type == GCONF_VALUE_BOOL)
-			panel_menu_button_set_use_custom_icon (button,
-							       gconf_value_get_bool (value));
+	if (g_strcmp0 (key, PANEL_MENU_BUTTON_MENU_PATH_KEY) == 0) {
+		value = g_settings_get_string (settings, key);
+		panel_menu_button_set_menu_path (button, value);
+	} else if (g_strcmp0 (key, PANEL_MENU_BUTTON_CUSTOM_ICON_KEY) == 0) {
+		value = g_settings_get_string (settings, key);
+		panel_menu_button_set_custom_icon (button, value);
+	} else if (g_strcmp0 (key, PANEL_MENU_BUTTON_TOOLTIP_KEY) == 0) {
+		value = g_settings_get_string (settings, key);
+		panel_menu_button_set_tooltip (button, value);
 	}
+
+	g_free (value);
 }
 
 static void
-panel_menu_button_connect_to_gconf (PanelMenuButton *button)
-{
-	GConfClient *client;
-	const char  *key;
-
-	client  = panel_gconf_get_client ();
-
-	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/objects/%s",
-				   button->priv->applet_id);
-	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	button->priv->gconf_notify =
-		gconf_client_notify_add (client, key,
-					 (GConfClientNotifyFunc) panel_menu_button_gconf_notify,
-					 button, NULL, NULL);
-}
-
-static void
-panel_menu_button_disconnect_from_gconf (PanelMenuButton *button)
-{
-	GConfClient *client;
-	const char  *key;
-
-	if (!button->priv->gconf_notify)
-		return;
-
-	client  = panel_gconf_get_client ();
-
-	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/objects/%s",
-				   button->priv->applet_id);
-
-	gconf_client_notify_remove (client, button->priv->gconf_notify);
-	button->priv->gconf_notify = 0;
-
-	gconf_client_remove_dir (client, key, NULL);
-}
-
-static void
-panel_menu_button_load (const char  *menu_path,
-			gboolean     use_menu_path,
-			const char  *custom_icon,
-			gboolean     use_custom_icon,
-			const char  *tooltip,
-			PanelWidget *panel,
-			gboolean     locked,
-			int          position,
-			gboolean     exactpos,
-			const char  *id)
+panel_menu_button_load_helper (const char  *menu_path,
+			       const char  *custom_icon,
+			       const char  *tooltip,
+			       PanelWidget *panel,
+			       const char  *id,
+			       GSettings   *settings)
 {
 	PanelMenuButton *button;
 	AppletInfo      *info;
@@ -653,35 +543,35 @@ panel_menu_button_load (const char  *menu_path,
 			       "menu-path", menu_path,
 			       "custom-icon", custom_icon,
 			       "tooltip", tooltip,
-			       "use-menu-path", use_menu_path,
-			       "use-custom-icon", use_custom_icon,
 			       "has-arrow", TRUE,
 			       NULL);
 
-	info = panel_applet_register (GTK_WIDGET (button), NULL, NULL,
-				      panel, locked, position, exactpos,
-				      PANEL_OBJECT_MENU, id);
+	info = panel_applet_register (GTK_WIDGET (button),
+				      panel,
+				      PANEL_OBJECT_MENU, id,
+				      settings,
+				      NULL, NULL);
 	if (!info) {
 		gtk_widget_destroy (GTK_WIDGET (button));
 		return;
 	}
 
-	button->priv->applet_id = g_strdup (info->id);
-
-	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"), NULL);
-
         if (panel_is_program_in_path ("alacarte") ||
 	    panel_is_program_in_path ("gmenu-simple-editor"))
 		panel_applet_add_callback (info, "edit", NULL,
-					   _("_Edit Menus"), NULL);
+					   _("_Edit Menus"),
+					   panel_lockdown_get_not_panels_locked_down_s);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
 
-	panel_menu_button_connect_to_gconf (button);
+	button->priv->settings_instance = panel_layout_get_instance_settings (
+							settings,
+							PANEL_MENU_BUTTON_SCHEMA);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_menu_button_recreate_menu),
-				   button);
+	g_signal_connect (button->priv->settings_instance, "changed",
+			  G_CALLBACK (panel_menu_button_settings_changed),
+			  button);
 }
 
 static char *
@@ -692,13 +582,11 @@ panel_menu_button_get_icon (PanelMenuButton *button)
 
 	retval = NULL;
 
-	if (button->priv->use_custom_icon &&
-	    button->priv->custom_icon)
+	if (!PANEL_GLIB_STR_EMPTY (button->priv->custom_icon))
 		retval = g_strdup (button->priv->custom_icon);
 
-	if (!retval                     &&
-	    button->priv->use_menu_path &&
-	    button->priv->menu_path     &&
+	if (!retval                                         &&
+	    !PANEL_GLIB_STR_EMPTY (button->priv->menu_path) &&
 	    panel_menu_button_create_menu (button)) {
 		directory = g_object_get_data (G_OBJECT (button->priv->menu),
 					       "panel-menu-tree-directory");
@@ -762,7 +650,15 @@ split_menu_uri (const char  *menu_uri,
 
 	return p;
 }
-                                                                                                             
+
+gboolean
+panel_menu_button_is_main_menu (PanelMenuButton *button)
+{
+	g_return_val_if_fail (PANEL_IS_MENU_BUTTON (button), FALSE);
+
+	return !PANEL_GLIB_STR_EMPTY (button->priv->menu_path);
+}
+
 void
 panel_menu_button_set_menu_path (PanelMenuButton *button,
 				 const char      *menu_uri)
@@ -822,110 +718,49 @@ panel_menu_button_set_tooltip (PanelMenuButton *button,
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
 
 	g_free (button->priv->tooltip);
-	button->priv->tooltip = NULL;
+	button->priv->tooltip = g_strdup (tooltip);
 
-	if (tooltip && tooltip [0]) {
-		button->priv->tooltip = g_strdup (tooltip);
-		panel_util_set_tooltip_text (GTK_WIDGET (button), tooltip);
-	}
+	panel_util_set_tooltip_text (GTK_WIDGET (button), tooltip);
 }
 
 void
-panel_menu_button_set_use_menu_path (PanelMenuButton *button,
-				     gboolean         use_menu_path)
+panel_menu_button_load (PanelWidget *panel,
+			const char  *id,
+			GSettings   *settings)
 {
-	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
-
-	use_menu_path = use_menu_path != FALSE;
-
-	if (button->priv->use_menu_path == use_menu_path)
-		return;
-
-	button->priv->use_menu_path = use_menu_path;
-
-	if (button->priv->menu)
-		gtk_menu_detach (GTK_MENU (button->priv->menu));
-	button->priv->menu = NULL;
-
-	panel_menu_button_set_icon (button);
-}
-
-gboolean
-panel_menu_button_get_use_menu_path (PanelMenuButton *button)
-{
-	g_return_val_if_fail (PANEL_IS_MENU_BUTTON (button), FALSE);
-
-	return button->priv->use_menu_path;
-}
-
-void
-panel_menu_button_set_use_custom_icon (PanelMenuButton *button,
-				       gboolean         use_custom_icon)
-{
-	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
-
-	button->priv->use_custom_icon = use_custom_icon != FALSE;
-
-	panel_menu_button_set_icon (button);
-}
-
-void
-panel_menu_button_load_from_gconf (PanelWidget *panel,
-				   gboolean     locked,
-				   int          position,
-				   gboolean     exactpos,
-				   const char  *id)
-{
-	GConfClient  *client;
+	GSettings    *settings_instance;
 	char         *scheme;
 	MenuPathRoot  root;
-	const char   *key;
 	char         *menu_path;
 	char         *custom_icon;
 	char         *tooltip;
-	gboolean      use_menu_path;
-	gboolean      use_custom_icon;
 
-	client  = panel_gconf_get_client ();
+	settings_instance = panel_layout_get_instance_settings (settings,
+								PANEL_MENU_BUTTON_SCHEMA);
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
-	menu_path = gconf_client_get_string (client, key, NULL);
+	menu_path = g_settings_get_string (settings_instance,
+					   PANEL_MENU_BUTTON_MENU_PATH_KEY);
+	custom_icon = g_settings_get_string (settings_instance,
+					     PANEL_MENU_BUTTON_CUSTOM_ICON_KEY);
+	tooltip = g_settings_get_string (settings_instance,
+					 PANEL_MENU_BUTTON_TOOLTIP_KEY);
 
 	scheme = g_strndup (menu_path, strcspn (menu_path, ":"));
 	root = panel_menu_scheme_to_path_root (scheme);
 	g_free (scheme);
-	if (root == SETTINGS_MENU) {
-		panel_compatiblity_migrate_settings_menu_button (client, id);
-		g_free (menu_path);
-		return;
-       }
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "custom_icon");
-	custom_icon = gconf_client_get_string (client, key, NULL);
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "tooltip");
-	tooltip = gconf_client_get_string (client, key, NULL);
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "use_menu_path");
-	use_menu_path = gconf_client_get_bool (client, key, NULL);
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "use_custom_icon");
-	use_custom_icon = gconf_client_get_bool (client, key, NULL);
-
-	panel_menu_button_load (menu_path,
-				use_menu_path,
-				custom_icon,
-				use_custom_icon,
-				tooltip,
-				panel,
-				locked,
-				position,
-				exactpos,
-				id);
+	panel_menu_button_load_helper (menu_path,
+				       custom_icon,
+				       tooltip,
+				       panel,
+				       id,
+				       settings);
 
 	g_free (menu_path);
 	g_free (custom_icon);
 	g_free (tooltip);
+
+	g_object_unref (settings_instance);
 }
 
 gboolean
@@ -933,46 +768,50 @@ panel_menu_button_create (PanelToplevel *toplevel,
 			  int            position,
 			  const char    *filename,
 			  const char    *menu_path,
-			  gboolean       use_menu_path,
 			  const char    *tooltip)
 {
-	GConfClient *client;
-	const char  *scheme;
-	const char  *key;
-	char        *id;
-
-	client  = panel_gconf_get_client ();
-
-	id = panel_profile_prepare_object (PANEL_OBJECT_MENU, toplevel, position, FALSE);
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "use_menu_path");
-	gconf_client_set_bool (client, key, use_menu_path, NULL);
+	char       *id;
+	GSettings  *settings;
+	GSettings  *settings_instance;
+	const char *scheme;
 
 	scheme = panel_menu_filename_to_scheme (filename);
 
 	if (filename && !scheme) {
 		g_warning ("Failed to find menu scheme for %s\n", filename);
-		g_free (id);
 		return FALSE;
 	}
 
-	if (use_menu_path && menu_path && menu_path [0] && scheme) {
-		char       *menu_uri;
+	id = panel_layout_object_create_start (PANEL_OBJECT_MENU, NULL,
+					       panel_toplevel_get_id (toplevel),
+					       position, FALSE,
+					       &settings);
+
+	settings_instance = panel_layout_get_instance_settings (settings,
+								PANEL_MENU_BUTTON_SCHEMA);
+
+	if (!PANEL_GLIB_STR_EMPTY (menu_path) && scheme) {
+		char *menu_uri;
 
 		menu_uri = g_strconcat (scheme, ":", menu_path, NULL);
 
-		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
-		gconf_client_set_string (client, key, menu_uri, NULL);
+		g_settings_set_string (settings_instance,
+				       PANEL_MENU_BUTTON_MENU_PATH_KEY,
+				       menu_uri);
 
 		g_free (menu_uri);
 	}
 
-	if (tooltip && tooltip [0]) {
-		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "tooltip");
-		gconf_client_set_string (client, key, tooltip, NULL);
+	if (!PANEL_GLIB_STR_EMPTY (tooltip)) {
+		g_settings_set_string (settings_instance,
+				       PANEL_MENU_BUTTON_TOOLTIP_KEY,
+				       tooltip);
 	}
 
-	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+	panel_layout_object_create_finish (id);
+
+	g_object_unref (settings_instance);
+	g_object_unref (settings);
 	g_free (id);
 
 	return TRUE;
