@@ -24,9 +24,10 @@
 
 #include <config.h>
 
-#include "panel-separator.h"
 #include "panel-background.h"
-#include "panel-profile.h"
+#include "panel-layout.h"
+
+#include "panel-separator.h"
 
 #define SEPARATOR_SIZE 10
 
@@ -228,21 +229,20 @@ panel_separator_set_orientation (PanelSeparator   *separator,
 }
 
 void
-panel_separator_load_from_gconf (PanelWidget *panel,
-				 gboolean     locked,
-				 int          position,
-				 const char  *id)
+panel_separator_load (PanelWidget *panel,
+		      const char  *id,
+		      GSettings   *settings)
 {
 	PanelSeparator *separator;
 
 	separator = g_object_new (PANEL_TYPE_SEPARATOR, NULL);
 
 	separator->priv->info = panel_applet_register (GTK_WIDGET (separator),
-						       NULL, NULL,
-						       panel, locked, position,
-						       TRUE,
+						       panel,
 						       PANEL_OBJECT_SEPARATOR,
-						       id);
+						       id, settings,
+						       NULL, NULL
+						       );
 
 	if (!separator->priv->info) {
 		gtk_widget_destroy (GTK_WIDGET (separator));
@@ -259,12 +259,9 @@ void
 panel_separator_create (PanelToplevel *toplevel,
 			int            position)
 {
-	char *id;
-
-	id = panel_profile_prepare_object (PANEL_OBJECT_SEPARATOR,
-					   toplevel, position, FALSE);
-	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
-	g_free (id);
+	panel_layout_object_create (PANEL_OBJECT_SEPARATOR, NULL,
+				    panel_toplevel_get_id (toplevel),
+				    position, FALSE);
 }
 
 void
