@@ -35,14 +35,13 @@
 #include <libpanel-util/panel-xdg.h>
 
 #include "applet.h"
-#include "nothing.h"
 #include "xstuff.h"
-#include "panel-config-global.h"
+#include "panel-bindings.h"
 #include "panel-gconf.h"
 #include "panel-globals.h"
 #include "launcher.h"
 #include "panel-icon-names.h"
-#include "panel-lockdown.h"
+#include "panel-schemas.h"
 
 char *
 panel_util_make_exec_uri_for_desktop (const char *exec)
@@ -334,140 +333,11 @@ panel_load_icon (GtkIconTheme  *icon_theme,
 	return retval;
 }
 
-static char *
-panel_lock_screen_action_get_command (const char *action)
-{
-	char    *command          = NULL;
-	gboolean use_gscreensaver = FALSE;
-
-	if (panel_is_program_in_path ("gnome-screensaver-command")
-	    && panel_is_program_in_path ("gnome-screensaver-preferences"))
-		use_gscreensaver = TRUE;
-	else if (!panel_is_program_in_path ("xscreensaver-command"))
-		return NULL;
-
-	if (strcmp (action, "prefs") == 0) {
-		if (use_gscreensaver) {
-			command = g_strdup ("gnome-screensaver-preferences");
-		} else if (panel_is_program_in_path ("xscreensaver-demo")) {
-			command = g_strdup ("xscreensaver-demo");
-		} else {
-			command = NULL;
-		}
-	} else if (strcmp (action, "activate") == 0
-		   || strcmp (action, "lock") == 0) {
-		/* Neither gnome-screensaver or xscreensaver allow root
-		 * to lock the screen */
-		if (geteuid () == 0) {
-			command = NULL;
-		} else {
-			if (use_gscreensaver) {
-				command = g_strdup_printf ("gnome-screensaver-command --%s", action);
-			} else {
-				command = g_strdup_printf ("xscreensaver-command -%s", action);
-			}
-		}
-	}
-
-	return command;
-}
-
-gboolean
-panel_lock_screen_action_available (const char *action)
-{
-	char    *command;
-	gboolean enabled = FALSE;
-
-	g_return_val_if_fail (action != NULL, FALSE);
-
-	if (strcmp (action, "prefs") != 0 &&
-	    panel_lockdown_get_disable_lock_screen ())
-		return FALSE;
-
-	command = panel_lock_screen_action_get_command (action);
-	if (command)
-		enabled = TRUE;
-
-	g_free (command);
-
-	return enabled;
-}
-
-void
-panel_lock_screen_action (GdkScreen  *screen,
-			  const char *action)
-{
-	GError  *error            = NULL;
-	char    *command          = NULL;
-	GdkAppLaunchContext *launch_context = NULL;
-	GAppInfo            *app_info = NULL;
-	GdkDisplay          *display;
-
-	g_return_if_fail (GDK_IS_SCREEN (screen));
-	g_return_if_fail (action != NULL);
-
-	if (strcmp (action, "prefs") != 0 &&
-	    panel_lockdown_get_disable_lock_screen ())
-		return;
-
-	command = panel_lock_screen_action_get_command (action);
-
-	if (!command)
-		return;
-
-	app_info = g_app_info_create_from_commandline (command,
-						       NULL,
-						       G_APP_INFO_CREATE_NONE,
-						       &error);
-
-	if (!error) {
-		display = gdk_screen_get_display (screen);
-		launch_context = gdk_display_get_app_launch_context (display);
-		gdk_app_launch_context_set_screen (launch_context, screen);
-		g_app_info_launch (app_info, NULL, G_APP_LAUNCH_CONTEXT (launch_context), &error);
-
-		g_object_unref (launch_context);
-	}
-
-	/* error either by create or launch the command */
-	if (error) {
-		char *primary;
-
-		primary = g_strdup_printf (_("Could not execute '%s'"),
-					   command);
-		panel_error_dialog (NULL, screen,
-				    "cannot_exec_screensaver", TRUE,
-				    primary, error->message);
-		g_free (primary);
-		g_error_free (error);
-	}
-
-	g_free (command);
-	g_object_unref (app_info);
-}
-
-void
-panel_lock_screen (GdkScreen *screen)
-{
-	panel_lock_screen_action (screen, "lock");
-}
-
-
-#define PANEL_LAUNCHER_PERSONAL_PATH "panel2.d/default/launchers"
-
 static char *
 panel_launcher_get_personal_path (void)
 {
-        const char *override;
-
-        override = g_getenv ("GNOME22_USER_DIR");
-
-        if (override)
-               return g_build_filename (override,
-                                        PANEL_LAUNCHER_PERSONAL_PATH, NULL);
-        else
-                return g_build_filename (g_get_home_dir (), ".gnome2",
-                                         PANEL_LAUNCHER_PERSONAL_PATH, NULL);
+	return g_build_filename (g_get_user_config_dir (),
+				 "gnome-panel", "launchers", NULL);
 }
 
 gboolean
@@ -1170,7 +1040,15 @@ panel_util_query_tooltip_cb (GtkWidget  *widget,
 			     GtkTooltip *tooltip,
 			     const char *text)
 {
-	if (!panel_global_config_get_tooltips_enabled ())
+	GSettings *gsettings;
+	gboolean   enable_tooltips;
+
+	gsettings = g_settings_new (PANEL_GENERAL_SCHEMA);
+	enable_tooltips = g_settings_get_boolean (gsettings,
+						  PANEL_GENERAL_ENABLE_TOOLTIPS_KEY);
+	g_object_unref (gsettings);
+
+	if (!enable_tooltips)
 		return FALSE;
 
 	gtk_tooltip_set_text (tooltip, text);
@@ -1224,3 +1102,77 @@ panel_util_get_file_optional_homedir (const char *location)
 
 	return file;
 }
+
+/*
+ * Keep code in panel-applet-bindings.c in sync!
+ */
+static void
+panel_util_key_event_is_binding (GdkEventKey *event,
+				 GType        type,
+				 const char  *signal_name,
+				 gboolean    *is_popup,
+				 gboolean    *is_popup_modifier)
+{
+	GtkBindingSet   *binding_set;
+	GtkBindingEntry *binding_entry;
+	gboolean         popup = FALSE;
+	gboolean         popup_modifier = FALSE;
+	char            *signal_dash;
+	char            *signal_underscore;
+
+	signal_dash = g_strdup (signal_name);
+	g_strdelimit (signal_dash, "_", '-');
+	signal_underscore = g_strdup (signal_name);
+	g_strdelimit (signal_underscore, "-", '_');
+
+	binding_set = gtk_binding_set_by_class (g_type_class_peek (type));
+
+	for (binding_entry = binding_set->entries;
+	     binding_entry != NULL;
+	     binding_entry = binding_entry->set_next) {
+		GtkBindingSignal *binding_signal;
+
+		for (binding_signal = binding_entry->signals;
+		     binding_signal != NULL;
+		     binding_signal = binding_signal->next) {
+			if (g_strcmp0 (binding_signal->signal_name, signal_dash) == 0 ||
+			    g_strcmp0 (binding_signal->signal_name, signal_underscore) == 0) {
+				if (binding_entry->keyval != event->keyval)
+					break;
+
+				popup = (event->state & GDK_MODIFIER_MASK) == binding_entry->modifiers;
+				popup_modifier = (event->state & GDK_MODIFIER_MASK) == (panel_bindings_get_mouse_button_modifier_keymask ()|binding_entry->modifiers);
+				break;
+			}
+		}
+
+		if (popup || popup_modifier)
+			break;
+	}
+
+	if (is_popup)
+		*is_popup = popup;
+	if (is_popup_modifier)
+		*is_popup_modifier = popup_modifier;
+
+	g_free (signal_dash);
+	g_free (signal_underscore);
+}
+
+void
+panel_util_key_event_is_popup (GdkEventKey *event,
+			       gboolean    *is_popup,
+			       gboolean    *is_popup_modifier)
+{
+	panel_util_key_event_is_binding (event, GTK_TYPE_WIDGET, "popup-menu",
+					 is_popup, is_popup_modifier);
+}
+
+void
+panel_util_key_event_is_popup_panel (GdkEventKey *event,
+				     gboolean    *is_popup,
+				     gboolean    *is_popup_modifier)
+{
+	panel_util_key_event_is_binding (event, PANEL_TYPE_TOPLEVEL, "popup-panel-menu",
+					 is_popup, is_popup_modifier);
+}
