@@ -27,8 +27,6 @@ G_BEGIN_DECLS
 
 #define PANEL_MINIMUM_WIDTH 12
 
-#define PANEL_APPLET_ASSOC_PANEL_KEY "panel_applet_assoc_panel_key"
-#define PANEL_APPLET_FORBIDDEN_PANELS "panel_applet_forbidden_panels"
 #define PANEL_APPLET_DATA "panel_applet_data"
 
 #ifndef TYPEDEF_PANEL_WIDGET
@@ -42,6 +40,19 @@ typedef struct _AppletRecord		AppletRecord;
 typedef struct _AppletData		AppletData;
 typedef struct _DNDRecord		DNDRecord;
 
+typedef struct _AppletSizeHints		AppletSizeHints;
+typedef struct _AppletSizeHintsAlloc	AppletSizeHintsAlloc;
+
+struct _AppletSizeHints {
+	int *hints;
+	int  len;
+};
+
+struct _AppletSizeHintsAlloc {
+	int index;
+	int size;
+};
+
 struct _AppletData
 {
 	GtkWidget *	applet;
@@ -64,8 +75,6 @@ struct _AppletData
 	guint           size_constrained : 1;
 	guint           expand_major : 1;
 	guint           expand_minor : 1;
-	guint           locked : 1;
-  
 };
 
 struct _PanelWidget
@@ -86,8 +95,6 @@ struct _PanelWidget
 
 	PanelBackground background;
 
-	GtkWidget      *master_widget;
-	
 	GtkWidget      *drop_widget;     /* widget that the panel checks for
 	                                  * the cursor on drops usually the
 	                                  * panel widget itself
@@ -97,6 +104,12 @@ struct _PanelWidget
 	
 	GdkEventKey    *key_event;
 
+	/* helpers to get a good size in packed panels with applets using
+	 * size hints */
+	int                   nb_applets_size_hints;
+	AppletSizeHints      *applets_hints;
+	AppletSizeHintsAlloc *applets_using_hint;
+
 	guint           packed : 1;
 };
 
@@ -133,13 +146,9 @@ GtkWidget *	panel_widget_new		(PanelToplevel  *toplevel,
   is on, we REALLY want to insert at the pos given by pos*/
 int		panel_widget_add		(PanelWidget *panel,
 						 GtkWidget   *applet,
-						 gboolean     locked,
 						 int          pos,
 						 gboolean     insert_at_pos);
 
-/*needs to be called for drawers after add*/
-void		panel_widget_add_forbidden	(PanelWidget *panel);
-
 /*move applet to a different panel*/
 int		panel_widget_reparent		(PanelWidget *old_panel,
 						 PanelWidget *new_panel,
@@ -176,7 +185,7 @@ void		panel_widget_draw_icon		(PanelWidget *panel,
 /*tells us if an applet is "stuck" on the right side*/
 int		panel_widget_is_applet_stuck	(PanelWidget *panel,
 						 GtkWidget *applet);
-/*get pos of the cursor location*/
+/*get pos of the cursor location in panel coordinates*/
 int		panel_widget_get_cursorloc	(PanelWidget *panel);
 
 /*needed for other panel types*/
@@ -200,14 +209,6 @@ void     panel_widget_set_applet_size_hints       (PanelWidget *panel,
 						   int         *size_hints,
 						   int          size_hints_len);
 
-void     panel_widget_set_applet_locked           (PanelWidget *panel,
-						   GtkWidget   *applet,
-						   gboolean     locked);
-gboolean panel_widget_get_applet_locked           (PanelWidget *panel,
-						   GtkWidget   *applet);
-gboolean panel_widget_toggle_applet_locked        (PanelWidget *panel,
-						   GtkWidget   *applet);
-
 void     panel_widget_register_open_dialog        (PanelWidget *panel,
 						   GtkWidget   *dialog);  
 G_END_DECLS
