@@ -25,26 +25,23 @@
 #include "panel.h"
 
 #include "applet.h"
-#include "drawer.h"
 #include "button-widget.h"
 #include "launcher.h"
 #include "panel-bindings.h"
 #include "panel-context-menu.h"
 #include "panel-util.h"
-#include "panel-config-global.h"
-#include "panel-gconf.h"
-#include "panel-profile.h"
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
 #include "panel-separator.h"
-#include "panel-compatibility.h"
 #include "panel-multiscreen.h"
 #include "panel-toplevel.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
 #include "panel-lockdown.h"
 #include "panel-icon-names.h"
+#include "panel-layout.h"
+#include "panel-schemas.h"
 
 enum {
 	TARGET_URL,
@@ -82,20 +79,6 @@ orientation_change (AppletInfo  *info,
 	case PANEL_OBJECT_MENU_BAR:
 		panel_menu_bar_set_orientation (PANEL_MENU_BAR (info->widget), orientation);
 		break;
-	case PANEL_OBJECT_DRAWER: {
-		Drawer      *drawer = info->data;
-		PanelWidget *panel_widget;
-
-		panel_widget = panel_toplevel_get_panel_widget (drawer->toplevel);
-
-		button_widget_set_orientation (BUTTON_WIDGET (info->widget), orientation);
-
-		gtk_widget_queue_resize (GTK_WIDGET (drawer->toplevel));
-		gtk_container_foreach (GTK_CONTAINER (panel_widget),
-				       orient_change_foreach,
-				       panel_widget);
-		}
-		break;
 	case PANEL_OBJECT_SEPARATOR:
 		panel_separator_set_orientation (PANEL_SEPARATOR (info->widget),
 						 orientation);
@@ -208,23 +191,6 @@ panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 	back_change(info,PANEL_WIDGET(widget));
 }
 
-static void
-panel_applet_removed(GtkWidget *widget, GtkWidget *applet, gpointer data)
-{
-	PanelToplevel *toplevel;
-	AppletInfo    *info;
-
-	toplevel = PANEL_WIDGET (widget)->toplevel;
-	info = g_object_get_data (G_OBJECT (applet), "applet_info");
-
-	if (info->type == PANEL_OBJECT_DRAWER) {
-		Drawer *drawer = info->data;
-
-		if (drawer->toplevel)
-			panel_toplevel_queue_auto_hide (toplevel);
-	}
-}
-
 static gboolean
 deactivate_idle (gpointer data)
 {
@@ -253,28 +219,10 @@ context_menu_show (GtkWidget *w,
 	panel_toplevel_push_autohide_disabler (PANEL_TOPLEVEL (pd->panel));
 }
 
-static void
-panel_recreate_context_menu (PanelData *pd)
-{
-	if (pd->menu) {
-		if (gtk_widget_get_visible (pd->menu))
-			gtk_menu_shell_deactivate (GTK_MENU_SHELL (pd->menu));
-
-		g_signal_handlers_disconnect_by_func (pd->menu,
-						      context_menu_deactivate,
-						      pd);
-		g_object_unref (pd->menu);
-	}
-	pd->menu = NULL;
-}
-
 static void
 panel_destroy (PanelToplevel *toplevel,
 	       PanelData     *pd)
 {
-	panel_lockdown_notify_remove (G_CALLBACK (panel_recreate_context_menu),
-				      pd);
-
 	if (pd->menu) {
 		g_signal_handlers_disconnect_by_func (pd->menu,
 						      context_menu_deactivate,
@@ -307,6 +255,25 @@ panel_applet_move(PanelWidget *panel, GtkWidget *widget, gpointer data)
 	panel_applet_save_position (info, info->id, FALSE);
 }
 
+static void
+panel_menu_lockdown_changed (PanelLockdown *lockdown,
+			     gpointer       user_data)
+{
+	PanelData *pd = user_data;
+
+	if (pd->menu) {
+		if (gtk_widget_get_visible (pd->menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL (pd->menu));
+
+		g_signal_handlers_disconnect_by_func (pd->menu,
+						      context_menu_deactivate,
+						      pd);
+
+		g_object_unref (pd->menu);
+		pd->menu = NULL;
+	}
+}
+
 static GtkWidget *
 panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
@@ -319,6 +286,12 @@ panel_menu_get (PanelWidget *panel, PanelData *pd)
 					  pd);
 			g_signal_connect (pd->menu, "show",
 					  G_CALLBACK (context_menu_show), pd);
+
+			panel_lockdown_on_notify (panel_lockdown_get (),
+						  NULL,
+						  G_OBJECT (pd->menu),
+						  panel_menu_lockdown_changed,
+						  pd);
 		}
 	}
 
@@ -414,23 +387,53 @@ panel_key_press_event (PanelToplevel *toplevel,
 	return FALSE;
 }
 
+static GSettings *
+get_settings_background_for_toplevel (PanelToplevel *toplevel)
+{
+	char      *toplevel_settings_path;
+	GSettings *settings;
+	GSettings *settings_background;
+
+	g_object_get (toplevel, "settings-path", &toplevel_settings_path, NULL);
+	settings = g_settings_new_with_path (PANEL_TOPLEVEL_SCHEMA,
+					     toplevel_settings_path);
+	settings_background = g_settings_get_child (settings,
+						    PANEL_BACKGROUND_SCHEMA_CHILD);
+
+	g_object_unref (settings);
+	g_free (toplevel_settings_path);
+
+	return settings_background;
+}
+
 static gboolean
 set_background_image_from_uri (PanelToplevel *toplevel,
 			       const char    *uri)
 {
-	char *image;
+	GFile     *file;
+	GSettings *settings;
 
-	if ( ! panel_profile_is_writable_background_type (toplevel) ||
-	     ! panel_profile_is_writable_background_image (toplevel))
+	file = g_file_new_for_uri (uri);
+	if (!g_file_is_native (file)) {
+		g_object_unref (file);
 		return FALSE;
+	}
+	g_object_unref (file);
 
-	if (!(image = g_filename_from_uri (uri, NULL, NULL)))
+	settings = get_settings_background_for_toplevel (toplevel);
+
+	if (!g_settings_is_writable (settings,
+				     PANEL_BACKGROUND_TYPE_KEY) ||
+	    !g_settings_is_writable (settings,
+				     PANEL_BACKGROUND_IMAGE_URI_KEY)) {
+		g_object_unref (settings);
 		return FALSE;
+	}
 
-	panel_profile_set_background_image (toplevel, image);
-	panel_profile_set_background_type (toplevel, PANEL_BACK_IMAGE);
+	g_settings_set_string (settings, PANEL_BACKGROUND_IMAGE_URI_KEY, uri);
+	g_settings_set_enum (settings, PANEL_BACKGROUND_TYPE_KEY, PANEL_BACK_IMAGE);
 
-	g_free (image);
+	g_object_unref (settings);
 
 	return FALSE;
 }
@@ -439,22 +442,55 @@ static gboolean
 set_background_color (PanelToplevel *toplevel,
 		      guint16       *dropped)
 {
-	GdkRGBA color;
+	GSettings *settings;
+	GdkRGBA    color;
+	char      *color_str;
 
 	if (!dropped)
 		return FALSE;
 
-	if ( ! panel_profile_is_writable_background_type (toplevel) ||
-	     ! panel_profile_is_writable_background_color (toplevel))
+	settings = get_settings_background_for_toplevel (toplevel);
+
+	if (!g_settings_is_writable (settings,
+				     PANEL_BACKGROUND_TYPE_KEY) ||
+	    !g_settings_is_writable (settings,
+				     PANEL_BACKGROUND_COLOR_KEY)) {
+		g_object_unref (settings);
 		return FALSE;
+	}
 
 	color.red   = dropped [0] / 65535.;
 	color.green = dropped [1] / 65535.;
 	color.blue  = dropped [2] / 65535.;
 	color.alpha = 1.;
 
-	panel_profile_set_background_color (toplevel, &color);
-	panel_profile_set_background_type (toplevel, PANEL_BACK_COLOR);
+	color_str = gdk_rgba_to_string (&color);
+
+	g_settings_set_string (settings, PANEL_BACKGROUND_COLOR_KEY, color_str);
+	g_settings_set_enum (settings, PANEL_BACKGROUND_TYPE_KEY, PANEL_BACK_COLOR);
+
+	g_free (color_str);
+	g_object_unref (settings);
+
+	return TRUE;
+}
+
+static gboolean
+reset_background (PanelToplevel *toplevel)
+{
+	GSettings *settings;
+
+	settings = get_settings_background_for_toplevel (toplevel);
+
+	if (!g_settings_is_writable (settings,
+				     PANEL_BACKGROUND_TYPE_KEY)) {
+		g_object_unref (settings);
+		return FALSE;
+	}
+
+	g_settings_set_enum (settings, PANEL_BACKGROUND_TYPE_KEY, PANEL_BACK_NONE);
+
+	g_object_unref (settings);
 
 	return TRUE;
 }
@@ -474,7 +510,7 @@ drop_url (PanelWidget *panel,
 
 	g_return_val_if_fail (url != NULL, FALSE);
 
-	if (!panel_profile_id_lists_are_writable ())
+	if (!panel_layout_is_writable ())
 		return FALSE;
 
 	netscape_url = g_strsplit (url, "\n", 2);
@@ -508,14 +544,13 @@ drop_menu (PanelWidget *panel,
 	   const char  *menu_filename,
 	   const char  *menu_path)
 {
-	if (!panel_profile_id_lists_are_writable ())
+	if (!panel_layout_is_writable ())
 		return FALSE;
 
 	return panel_menu_button_create (panel->toplevel,
 					 position,
 					 menu_filename,
 					 menu_path,
-					 menu_path != NULL,
 					 NULL);
 
 }
@@ -532,7 +567,7 @@ drop_uri (PanelWidget *panel,
 	char  *icon;
 	GFile *file;
 
-	if (!panel_profile_id_lists_are_writable ())
+	if (!panel_layout_is_writable ())
 		return FALSE;
 
 	name = panel_util_get_label_for_uri (uri);
@@ -578,7 +613,7 @@ drop_nautilus_desktop_uri (PanelWidget *panel,
 			return FALSE;
 
 	success = TRUE;
-	id = panel_profile_get_toplevel_id (panel->toplevel);
+	id = panel_toplevel_get_id (panel->toplevel);
 	basename = uri + strlen ("x-nautilus-desktop:///");
 
 	if (strncmp (basename, "trash", strlen ("trash")) == 0)
@@ -719,7 +754,7 @@ drop_urilist (PanelWidget *panel,
 				   (!strcmp (mime, "application/x-gnome-app-info") ||
 				    !strcmp (mime, "application/x-desktop") ||
 				    !strcmp (mime, "application/x-kde-app-info"))) {
-				if (panel_profile_id_lists_are_writable ())
+				if (panel_layout_is_writable ())
 					panel_launcher_create (panel->toplevel, pos, uri);
 				else
 					success = FALSE;
@@ -728,11 +763,11 @@ drop_urilist (PanelWidget *panel,
 
 				filename = g_file_get_path (file);
 
-				if (panel_profile_id_lists_are_writable ())
+				if (panel_layout_is_writable ())
 					/* executable and local, so add a
 					 * launcher with it */
 					ask_about_launcher (filename, panel,
-							    pos, TRUE);
+							    pos);
 				else
 					success = FALSE;
 				g_free (filename);
@@ -770,7 +805,7 @@ drop_internal_icon (PanelWidget *panel,
 	if (!icon_name)
 		return FALSE;
 
-	if (!panel_profile_id_lists_are_writable ())
+	if (!panel_layout_is_writable ())
 		return FALSE;
 
 	if (action == GDK_ACTION_MOVE)
@@ -780,12 +815,16 @@ drop_internal_icon (PanelWidget *panel,
 		return FALSE;
 
 	if (old_launcher && old_launcher->button) {
+		const char *object_id;
+
 		if (old_launcher->prop_dialog) {
 			g_signal_handler_disconnect (old_launcher->button,
 						     old_launcher->destroy_handler);
 			launcher_properties_destroy (old_launcher);
 		}
-		panel_profile_delete_object (old_launcher->info);
+
+		object_id = panel_applet_get_id (old_launcher->info);
+		panel_layout_delete_object (object_id);
 	}
 
 	return TRUE;
@@ -814,14 +853,10 @@ move_applet (PanelWidget *panel, int pos, int applet_index)
 	    info->widget != NULL &&
 	    parent != NULL &&
 	    PANEL_IS_WIDGET (parent)) {
-		GSList *forb;
-		forb = g_object_get_data (G_OBJECT (info->widget),
-					  PANEL_APPLET_FORBIDDEN_PANELS);
-		if ( ! g_slist_find (forb, panel))
-			panel_widget_reparent (PANEL_WIDGET (parent),
-					       panel,
-					       info->widget,
-					       pos);
+		panel_widget_reparent (PANEL_WIDGET (parent),
+				       panel,
+				       info->widget,
+				       pos);
 	}
 
 	return TRUE;
@@ -838,10 +873,9 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 	if (applet_type == NULL)
 		return FALSE;
 
-	if (sscanf (applet_type, "MENU:%d", &applet_index) == 1 ||
-	    sscanf (applet_type, "DRAWER:%d", &applet_index) == 1) {
+	if (sscanf (applet_type, "MENU:%d", &applet_index) == 1) {
 		if (action != GDK_ACTION_MOVE)
-			g_warning ("Only MOVE supported for menus/drawers");
+			g_warning ("Only MOVE supported for menus");
 		success = move_applet (panel, pos, applet_index);
 
 	} else if (strncmp (applet_type, "MENU:", strlen ("MENU:")) == 0) {
@@ -866,16 +900,8 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 			g_free (menu_filename);
 		}
 
-	} else if (!strcmp (applet_type, "DRAWER:NEW")) {
-		if (panel_profile_id_lists_are_writable ()) {
-			panel_drawer_create (panel->toplevel, pos, NULL, FALSE, NULL);
-			success = TRUE;
-		} else {
-			success = FALSE;
-		}
-
 	} else if (!strncmp (applet_type, "ACTION:", strlen ("ACTION:"))) {
-		if (panel_profile_id_lists_are_writable ()) {
+		if (panel_layout_is_writable ()) {
 			remove_applet = panel_action_button_load_from_drag (
 							panel->toplevel,
 							pos,
@@ -887,7 +913,7 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		}
 
 	} else if (!strcmp (applet_type, "MENUBAR:NEW")) {
-		if (panel_profile_id_lists_are_writable ()) {
+		if (panel_layout_is_writable ()) {
 			panel_menu_bar_create (panel->toplevel, pos);
 			success = TRUE;
 		} else {
@@ -895,7 +921,7 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		}
 
 	} else if (!strcmp(applet_type,"SEPARATOR:NEW")) {
-		if (panel_profile_id_lists_are_writable ()) {
+		if (panel_layout_is_writable ()) {
 			panel_separator_create (panel->toplevel, pos);
 			success = TRUE;
 		} else {
@@ -903,8 +929,8 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		}
 
 	} else if (!strcmp(applet_type,"LAUNCHER:ASK")) {
-		if (panel_profile_id_lists_are_writable ()) {
-			ask_about_launcher (NULL, panel, pos, TRUE);
+		if (panel_layout_is_writable ()) {
+			ask_about_launcher (NULL, panel, pos);
 			success = TRUE;
 		} else {
 			success = FALSE;
@@ -921,7 +947,7 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		info = g_slist_nth_data (applet_list, applet_index);
 
 		if (info)
-			panel_profile_delete_object (info);
+			panel_layout_delete_object (panel_applet_get_id (info));
 	}
 
 	return success;
@@ -1025,25 +1051,9 @@ panel_check_drop_forbidden (PanelWidget    *panel,
 	if (!panel)
 		return FALSE;
 
-	if (panel_lockdown_get_locked_down ())
+	if (panel_lockdown_get_panels_locked_down_s ())
 		return FALSE;
 
-	if (info == TARGET_APPLET_INTERNAL) {
-		GtkWidget *source_widget;
-
-		source_widget = gtk_drag_get_source_widget (context);
-
-		if (BUTTON_IS_WIDGET (source_widget)) {
-			GSList *forb;
-
-			forb = g_object_get_data (G_OBJECT (source_widget),
-						  PANEL_APPLET_FORBIDDEN_PANELS);
-
-			if (g_slist_find (forb, panel))
-				return FALSE;
-		}
-	}
-
 	if (info == TARGET_ICON_INTERNAL ||
 	    info == TARGET_APPLET_INTERNAL) {
 		if (gdk_drag_context_get_actions (context) & GDK_ACTION_MOVE)
@@ -1136,7 +1146,7 @@ panel_receive_dnd_data (PanelWidget      *panel,
 	const guchar *data;
 	gboolean      success = FALSE;
 
-	if (panel_lockdown_get_locked_down ()) {
+	if (panel_lockdown_get_panels_locked_down_s ()) {
 		gtk_drag_finish (context, FALSE, FALSE, time_);
 		return;
 	}
@@ -1157,12 +1167,7 @@ panel_receive_dnd_data (PanelWidget      *panel,
 		success = set_background_image_from_uri (panel->toplevel, (char *) data);
 		break;
 	case TARGET_BACKGROUND_RESET:
-		if (panel_profile_is_writable_background_type (panel->toplevel)) {
-			panel_profile_set_background_type (panel->toplevel, PANEL_BACK_NONE);
-			success = TRUE;
-		} else {
-			success = FALSE;
-		}
+		success = reset_background (panel->toplevel);
 		break;
 	case TARGET_DIRECTORY:
 		success = drop_uri (panel, pos, (char *)data,
@@ -1173,7 +1178,7 @@ panel_receive_dnd_data (PanelWidget      *panel,
 			gtk_drag_finish (context, FALSE, FALSE, time_);
 			return;
 		}
-		if (panel_profile_id_lists_are_writable ()) {
+		if (panel_layout_is_writable ()) {
 			panel_applet_frame_create (panel->toplevel, pos, (char *) data);
 			success = TRUE;
 		} else {
@@ -1243,10 +1248,6 @@ panel_widget_setup(PanelWidget *panel)
 			  "applet_added",
 			  G_CALLBACK(panel_applet_added),
 			  NULL);
-	g_signal_connect (G_OBJECT(panel),
-			  "applet_removed",
-			  G_CALLBACK(panel_applet_removed),
-			  NULL);
 	g_signal_connect (G_OBJECT(panel),
 			  "applet_move",
 			  G_CALLBACK(panel_applet_move),
@@ -1281,9 +1282,6 @@ panel_setup (PanelToplevel *toplevel)
 	
 	g_object_set_data (G_OBJECT (toplevel), "PanelData", pd);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_recreate_context_menu),
-				   pd);
-
 	panel_widget_setup (panel_widget);
 
 	g_signal_connect (toplevel, "drag_data_received",
@@ -1342,20 +1340,7 @@ panel_is_applet_right_stick (GtkWidget *applet)
 static void
 panel_delete_without_query (PanelToplevel *toplevel)
 {
-	PanelWidget *panel_widget;
-
-	panel_widget = panel_toplevel_get_panel_widget (toplevel);
-
-	if (panel_toplevel_get_is_attached (toplevel) &&
-	    panel_widget->master_widget) {
-		AppletInfo *info;
-
-		info = g_object_get_data (G_OBJECT (panel_widget->master_widget),
-					  "applet_info");
-
-		panel_profile_delete_object (info);
-	} else
-		panel_profile_delete_toplevel (toplevel);
+	panel_layout_delete_toplevel (panel_toplevel_get_id (toplevel));
 } 
 
 static void
@@ -1385,28 +1370,19 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 {
 
 	GtkWidget *dialog;
-	char *text1;
-	char *text2;
-
-	if (panel_toplevel_get_is_attached (toplevel)) {
-		text1 = _("Delete this drawer?");
-		text2 = _("When a drawer is deleted, the drawer and its\n"
-			 "settings are lost.");
-	} else {
-		text1 = _("Delete this panel?");
-		text2 = _("When a panel is deleted, the panel and its\n"
-			 "settings are lost.");
-	}
 
 	dialog = gtk_message_dialog_new (
 			GTK_WINDOW (toplevel),
 			GTK_DIALOG_MODAL,
 			GTK_MESSAGE_WARNING,
 			GTK_BUTTONS_NONE,
-			"%s", text1);
+			"%s", _("Delete this panel?"));
 	
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-	                                          "%s", text2);	
+	                                          "%s",
+						  _("When a panel is deleted, the panel and its\n"
+						  "settings are lost."));
+
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
@@ -1458,11 +1434,18 @@ panel_query_deletion (PanelToplevel *toplevel)
 void
 panel_delete (PanelToplevel *toplevel)
 {
+	GSettings   *settings;
+	gboolean     confirm;
 	PanelWidget *panel_widget;
 
+	settings = g_settings_new (PANEL_GENERAL_SCHEMA);
+	confirm = g_settings_get_boolean (settings,
+					  PANEL_GENERAL_CONFIRM_PANEL_REMOVAL_KEY);
+	g_object_unref (settings);
+
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 
-	if (!panel_global_config_get_confirm_panel_remove () ||
+	if (!confirm ||
 	    !g_list_length (panel_widget->applet_list)) {
 		panel_delete_without_query (toplevel);
 		return;
