@@ -25,7 +25,20 @@
 #include <gconf/gconf-client.h>
 
 #include "panel-applet-gconf.h"
-#include "panel-applet-private.h"
+
+/**
+ * SECTION:gconf
+ * @short_description: utility API to use GConf with applets.
+ * @stability: Unstable
+ *
+ * The <function>panel_applet_gconf_*()</function> set of API provides
+ * convenience functions to access GConf keys that are specific to an
+ * applet instance.
+ *
+ * Keep in mind that it might be worth considering using <link
+ * linkend="getting-started.settings">global settings</link> for your applet,
+ * instead of settings specific to an instance.
+ */
 
 static GConfClient *
 panel_applet_gconf_get_client (void)
@@ -38,6 +51,16 @@ panel_applet_gconf_get_client (void)
 	return client;
 }
 
+/**
+ * panel_applet_gconf_get_full_key:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ *
+ * Returns the full GConf path of @key, in the per-instance GConf directory of
+ * @applet. The string should be freed by the caller.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 gchar *
 panel_applet_gconf_get_full_key (PanelApplet *applet,
 				 const gchar *key)
@@ -59,24 +82,29 @@ panel_applet_gconf_get_full_key (PanelApplet *applet,
 	return full_key;
 }
 
+/**
+ * panel_applet_gconf_set_bool:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @the_bool: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_bool() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_bool (PanelApplet  *applet,
 			     const gchar  *key,
 			     gboolean      the_bool,
-			     GError      **opt_error)
+			     GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -84,31 +112,31 @@ panel_applet_gconf_set_bool (PanelApplet  *applet,
 	gconf_client_set_bool (client, full_key, the_bool, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_set_int:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @the_int: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_int() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_int (PanelApplet  *applet,
 			    const gchar  *key,
 			    gint          the_int,
-			    GError      **opt_error)
+			    GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -116,31 +144,31 @@ panel_applet_gconf_set_int (PanelApplet  *applet,
 	gconf_client_set_int (client, full_key, the_int, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_set_string:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @the_string: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_string() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_string (PanelApplet  *applet,
 			       const gchar  *key,
 			       const gchar  *the_string,
-			       GError      **opt_error)
+			       GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -148,31 +176,31 @@ panel_applet_gconf_set_string (PanelApplet  *applet,
 	gconf_client_set_string (client, full_key, the_string, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_set_float:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @the_float: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_float() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_float (PanelApplet  *applet,
 			      const gchar  *key,
 			      gdouble       the_float,
-			      GError      **opt_error)
+			      GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -180,32 +208,33 @@ panel_applet_gconf_set_float (PanelApplet  *applet,
 	gconf_client_set_float (client, full_key, the_float, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_set_list:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @list_type: type of items in @list.
+ * @list: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_list() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_list (PanelApplet     *applet,
 			     const gchar     *key,
 			     GConfValueType   list_type,
 			     GSList          *list,
-			     GError         **opt_error)
+			     GError         **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -213,31 +242,31 @@ panel_applet_gconf_set_list (PanelApplet     *applet,
 	gconf_client_set_list (client, full_key, list_type, list, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_set_value:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @value: new value for @key.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_set_value() to update @key in the
+ * per-instance GConf directory of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_gconf_set_value (PanelApplet  *applet,
 			      const gchar  *key,
 			      GConfValue   *value,
-			      GError      **opt_error)
+			      GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -245,31 +274,32 @@ panel_applet_gconf_set_value (PanelApplet  *applet,
 	gconf_client_set (client, full_key, value, error);
 
 	g_free (full_key);
-
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_gconf_get_bool:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_bool() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 gboolean
 panel_applet_gconf_get_bool (PanelApplet  *applet,
 			     const gchar  *key,
-			     GError      **opt_error)
+			     GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	gboolean      retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), FALSE);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -278,32 +308,33 @@ panel_applet_gconf_get_bool (PanelApplet  *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
 
+/**
+ * panel_applet_gconf_get_int:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_int() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 gint
 panel_applet_gconf_get_int (PanelApplet  *applet,
 			    const gchar  *key,
-			    GError      **opt_error)
+			    GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	gint          retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), -1);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -312,32 +343,33 @@ panel_applet_gconf_get_int (PanelApplet  *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
 
+/**
+ * panel_applet_gconf_get_string:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_string() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key. The string should be freed by the caller.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 gchar *
 panel_applet_gconf_get_string (PanelApplet  *applet,
 			       const gchar  *key,
-			       GError      **opt_error)
+			       GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	gchar        *retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -346,32 +378,33 @@ panel_applet_gconf_get_string (PanelApplet  *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
 
+/**
+ * panel_applet_gconf_get_float:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_float() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 gdouble
 panel_applet_gconf_get_float (PanelApplet  *applet,
 			      const gchar  *key,
-			      GError      **opt_error)
+			      GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	gdouble       retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), 0.0);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -380,32 +413,33 @@ panel_applet_gconf_get_float (PanelApplet  *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
 
+/**
+ * panel_applet_gconf_get_value:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_value() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 GConfValue *
 panel_applet_gconf_get_value (PanelApplet  *applet,
 			      const gchar  *key,
-			      GError      **opt_error)
+			      GError      **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	GConfValue   *retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -414,33 +448,35 @@ panel_applet_gconf_get_value (PanelApplet  *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
 
+/**
+ * panel_applet_gconf_get_list:
+ * @applet: a #PanelApplet.
+ * @key: a GConf key name.
+ * @error: a #GError, or %NULL.
+ *
+ * Convenience wrapper around gconf_client_get_list() to get the value of @key
+ * in the per-instance GConf directory of @applet.
+ *
+ * Returns: the value of @key. The list and its content should be freed by the
+ * caller.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 GSList *
 panel_applet_gconf_get_list (PanelApplet     *applet,
 			     const gchar     *key,
 			     GConfValueType   list_type,
-			     GError         **opt_error)
+			     GError         **error)
 {
 	GConfClient  *client;
 	gchar        *full_key;
 	GSList       *retval;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
-	if (opt_error)
-		error = opt_error;
-	else
-		error = &our_error;
-
 	full_key = panel_applet_gconf_get_full_key (applet, key);
 
 	client = panel_applet_gconf_get_client ();
@@ -449,10 +485,5 @@ panel_applet_gconf_get_list (PanelApplet     *applet,
 
 	g_free (full_key);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": gconf error : '%s'", our_error->message);
-		g_error_free (our_error);
-	}
-
 	return retval;
 }
