@@ -30,11 +30,28 @@
 
 G_BEGIN_DECLS
 
+/**
+ * PanelAppletOrient:
+ * @PANEL_APPLET_ORIENT_UP: The #PanelApplet is on a horizontal panel, at the
+ * bottom of the screen. It is oriented towards the top of the screen.
+ * @PANEL_APPLET_ORIENT_DOWN: The #PanelApplet is on a horizontal panel, at the
+ * top of the screen. It is oriented towards the bottom of the screen.
+ * @PANEL_APPLET_ORIENT_LEFT: The #PanelApplet is on a vertical panel, at the
+ * right of the screen. It is oriented towards the left of the screen.
+ * @PANEL_APPLET_ORIENT_RIGHT: The #PanelApplet is on a vertical panel, at the
+ * left of the screen. It is oriented towards the right of the screen.
+ *
+ * Type defining the orientation of the applet. The values may seem backward
+ * (e.g. %PANEL_APPLET_ORIENT_RIGHT means the panel is on the left side), but
+ * this represents the direction the applet is oriented to.
+ **/
 typedef enum {
 	PANEL_APPLET_ORIENT_UP,
 	PANEL_APPLET_ORIENT_DOWN,
 	PANEL_APPLET_ORIENT_LEFT,
 	PANEL_APPLET_ORIENT_RIGHT
+#define PANEL_APPLET_ORIENT_FIRST PANEL_APPLET_ORIENT_UP
+#define PANEL_APPLET_ORIENT_LAST  PANEL_APPLET_ORIENT_RIGHT
 } PanelAppletOrient;
 
 #define PANEL_TYPE_APPLET         (panel_applet_get_type ())
@@ -44,23 +61,49 @@ typedef enum {
 #define PANEL_IS_APPLET_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), PANEL_TYPE_APPLET))
 #define PANEL_APPLET_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PANEL_TYPE_APPLET, PanelAppletClass))
 
-typedef enum {
-	PANEL_NO_BACKGROUND,
-	PANEL_COLOR_BACKGROUND,
-	PANEL_PIXMAP_BACKGROUND
-} PanelAppletBackgroundType;
-
+/**
+ * PanelAppletFlags:
+ * @PANEL_APPLET_FLAGS_NONE: No flags set.
+ * @PANEL_APPLET_EXPAND_MAJOR: On horizontal panels, the #PanelApplet will be
+ * allocated as much width as possible. On vertical panels, it will be
+ * allocated as much height as possible.
+ * @PANEL_APPLET_EXPAND_MINOR: On horizontal panels, the #PanelApplet will be
+ * allocated the full height of the panel. On vertical panels, it will be
+ * allocated the full width. Most applets should set this flag in order to use
+ * the full panel size and allow the applet to be Fitt's Law compliant.
+ * @PANEL_APPLET_HAS_HANDLE: The panel will draw a handle for the
+ * #PanelApplet. This handle will be usable to move the applet, and to open the
+ * context menu of the applet.
+ *
+ * Flags to be used with panel_applet_get_flags()/panel_applet_set_flags(), to
+ * indicate to the panel a specific behavior requested by the #PanelApplet.
+ **/
 typedef enum {
 	PANEL_APPLET_FLAGS_NONE   = 0,
 	PANEL_APPLET_EXPAND_MAJOR = 1 << 0,
 	PANEL_APPLET_EXPAND_MINOR = 1 << 1,
 	PANEL_APPLET_HAS_HANDLE   = 1 << 2
+#define PANEL_APPLET_FLAGS_ALL (PANEL_APPLET_EXPAND_MAJOR|PANEL_APPLET_EXPAND_MINOR|PANEL_APPLET_HAS_HANDLE)
 } PanelAppletFlags;
 
 typedef struct _PanelApplet        PanelApplet;
 typedef struct _PanelAppletClass   PanelAppletClass;
 typedef struct _PanelAppletPrivate PanelAppletPrivate;
 
+/**
+ * PanelAppletFactoryCallback:
+ * @applet: a newly-created #PanelApplet.
+ * @iid: identifier of the requested applet type.
+ * @user_data: user data.
+ *
+ * The type used for callbacks after. The callback will usually check that @iid
+ * is a valid applet type identifier for the applet factory, and will then fill
+ * @applet with widgets, connect to signals, etc.
+ *
+ * Returns: %TRUE if @iid is a valid applet type identifier for the applet
+ * factory and if the creation of @applet was successfully completed, %FALSE
+ * otherwise.
+ **/
 typedef gboolean (*PanelAppletFactoryCallback) (PanelApplet *applet,
 						const gchar *iid,
 						gpointer     user_data);
@@ -68,6 +111,7 @@ typedef gboolean (*PanelAppletFactoryCallback) (PanelApplet *applet,
 struct _PanelApplet {
 	GtkEventBox          event_box;
 
+	/*< private >*/
 	PanelAppletPrivate  *priv;
 };
 
@@ -80,34 +124,29 @@ struct _PanelAppletClass {
 	void (*change_size)   (PanelApplet       *applet,
 			       guint              size);
 
-	void (*change_background) (PanelApplet               *applet,
-				   PanelAppletBackgroundType  type,
-				   GdkColor                  *color,
-				   GdkPixmap                 *pixmap);
+	void (*change_background) (PanelApplet     *applet,
+				   cairo_pattern_t *pattern);
 	void (*move_focus_out_of_applet) (PanelApplet        *frame,
 					  GtkDirectionType    direction);
 };
 
 GType              panel_applet_get_type             (void) G_GNUC_CONST;
  
-GtkWidget         *panel_applet_new                  (void);
-
 PanelAppletOrient  panel_applet_get_orient           (PanelApplet *applet);
 guint              panel_applet_get_size             (PanelApplet *applet);
-PanelAppletBackgroundType
-                   panel_applet_get_background       (PanelApplet *applet,
-						      /* return values */
-						      GdkColor    *color,
-						      GdkPixmap  **pixmap);
+cairo_pattern_t   *panel_applet_get_background       (PanelApplet *applet);
 
 void               panel_applet_set_background_widget (PanelApplet *applet,
 						       GtkWidget   *widget);
 
+GSettings         *panel_applet_settings_new         (PanelApplet *applet,
+						      const char  *schema);
+
 gchar             *panel_applet_get_preferences_key  (PanelApplet *applet);
 
 void               panel_applet_add_preferences      (PanelApplet  *applet,
 						      const gchar  *schema_dir,
-						      GError      **opt_error);
+						      GError      **error);
 
 PanelAppletFlags   panel_applet_get_flags            (PanelApplet      *applet);
 void      	   panel_applet_set_flags            (PanelApplet      *applet,
@@ -131,11 +170,14 @@ void               panel_applet_setup_menu_from_file (PanelApplet        *applet
 						      GtkActionGroup     *action_group);
 
 int                panel_applet_factory_main          (const gchar		  *factory_id,
-						       gboolean                    out_process,
 						       GType                       applet_type,
 						       PanelAppletFactoryCallback  callback,
 						       gpointer			   data);
-gboolean           _panel_applet_shlib_factory        (void);
+
+int                panel_applet_factory_setup_in_process (const gchar               *factory_id,
+							  GType                      applet_type,
+							  PanelAppletFactoryCallback callback,
+							  gpointer                   data);
 
 
 /*
@@ -160,7 +202,29 @@ gboolean           _panel_applet_shlib_factory        (void);
 	} while (0)
 #endif /* !defined(ENABLE_NLS) */
 
-#define PANEL_APPLET_OUT_PROCESS_FACTORY(id, type, name, callback, data)	\
+/**
+ * PANEL_APPLET_OUT_PROCESS_FACTORY:
+ * @id: identifier of an applet factory.
+ * @type: GType of the applet this factory creates.
+ * @callback: (scope call): callback to be called when a new applet is created.
+ * @data: (closure): callback data.
+ *
+ * Convenience macro providing a main() function for an out-of-process applet.
+ * Internally, it will call panel_applet_factory_main() to create the
+ * @factory_id applet factory.
+ *
+ * Applet instances created by the applet factory will use @applet_type as
+ * GType. Unless you subclass #PanelApplet, you should use %PANEL_TYPE_APPLET
+ * as @applet_type.
+ *
+ * On creation of the applet instances, @callback is called to setup the
+ * applet. If @callback returns %FALSE, the creation of the applet instance is
+ * cancelled.
+ *
+ * It can only be used once, and is incompatible with the use of
+ * %PANEL_APPLET_IN_PROCESS_FACTORY and panel_applet_factory_main().
+ **/
+#define PANEL_APPLET_OUT_PROCESS_FACTORY(id, type, callback, data)		\
 int main (int argc, char *argv [])						\
 {										\
 	GOptionContext *context;						\
@@ -186,18 +250,41 @@ int main (int argc, char *argv [])						\
 										\
 	gtk_init (&argc, &argv);						\
                                                                                 \
-        retval = panel_applet_factory_main (id, TRUE, type, callback, data);    \
+        retval = panel_applet_factory_main (id, type, callback, data);          \
 	g_option_context_free (context);					\
 										\
 	return retval;								\
 }
 
-#define PANEL_APPLET_IN_PROCESS_FACTORY(id, type, descr, callback, data)	\
+/**
+ * PANEL_APPLET_IN_PROCESS_FACTORY:
+ * @id: identifier of an applet factory.
+ * @type: GType of the applet this factory creates.
+ * @callback: (scope call): callback to be called when a new applet is created.
+ * @data: (closure): callback data.
+ *
+ * Convenience macro providing the symbol needed to automatically register the
+ * @factory_id applet factory for an in-process applet.
+ *
+ * Applet instances created by the applet factory will use @applet_type as
+ * GType. Unless you subclass #PanelApplet, you should use %PANEL_TYPE_APPLET
+ * as @applet_type.
+ *
+ * On creation of the applet instances, @callback is called to setup the
+ * applet. If @callback returns %FALSE, the creation of the applet instance is
+ * cancelled.
+ *
+ * It can only be used once, and is incompatible with the use of
+ * %PANEL_APPLET_OUT_PROCESS_FACTORY and panel_applet_factory_main().
+ **/
+#define PANEL_APPLET_IN_PROCESS_FACTORY(id, type, callback, data)		\
+gboolean _panel_applet_shlib_factory (void);					\
 G_MODULE_EXPORT gint                                                            \
 _panel_applet_shlib_factory (void)					        \
 {										\
 	_PANEL_APPLET_SETUP_GETTEXT (FALSE);					\
-        return panel_applet_factory_main (id, FALSE, type, callback, data);     \
+        return panel_applet_factory_setup_in_process (id, type,                 \
+                                                      callback, data);          \
 }
 
 G_END_DECLS
